@@ -18,19 +18,20 @@
 
 #include "gdafe.h"
 
-static void add_environ_cb (GtkButton *button, gpointer data);
-static void clear_text_cb (GtkWidget *w, gpointer data);
-static void current_environ_cb (GtkButton *button, gpointer data);
-static void dialog_clicked_cb (GnomeDialog * dialog, gint button_number, gpointer data);
-static void font_set_cb (GnomeFontPicker *fontpicker, gchar *font_name, gpointer user_data);
-static void remove_environ_cb (GtkButton *button, gpointer data);
-static void save_as_cb (GtkWidget *w, gpointer data);
-static void save_config_cb (GtkWidget *w, gpointer data);
-static void save_dsn_cb (GtkWidget *w, gpointer data);
-static void save_on_exit_cb (GtkCheckButton *button, gpointer data);
-static void set_language_cb (GtkEntry *entry, gpointer data);
-static void set_shell_cb (GtkEntry *entry, gpointer data);
-static void set_tabs_cb (GtkToggleButton *button, gpointer data);
+static void add_environ_cb      (GtkButton *button, gpointer data);
+static void clear_text_cb       (GtkWidget *w, gpointer data);
+static void connect_on_entry_cb (GtkCheckButton *button, gpointer data);
+static void current_environ_cb  (GtkButton *button, gpointer data);
+static void dialog_clicked_cb   (GnomeDialog * dialog, gint button_number, gpointer data);
+static void font_set_cb         (GnomeFontPicker *fontpicker, gchar *font_name, gpointer user_data);
+static void remove_environ_cb   (GtkButton *button, gpointer data);
+static void save_as_cb          (GtkWidget *w, gpointer data);
+static void save_config_cb      (GtkWidget *w, gpointer data);
+static void save_dsn_cb         (GtkWidget *w, gpointer data);
+static void save_on_exit_cb     (GtkCheckButton *button, gpointer data);
+static void set_language_cb     (GtkEntry *entry, gpointer data);
+static void set_shell_cb        (GtkEntry *entry, gpointer data);
+static void set_tabs_cb         (GtkToggleButton *button, gpointer data);
 
 static GtkWidget* config_create_tab (void);
 static GtkWidget* dsn_create_tab (void);
@@ -205,16 +206,24 @@ general_create_tab (void)
   
   /* on/off options */
   box = gtk_vbox_new(FALSE, 0);
-  button = gda_ui_new_check_button_widget(_("Save options on exit"), 1);
+  button = gda_ui_new_check_button_widget(_("Save options on exit"),
+                                          gnome_config_get_int(FE_CONFIG_GENERAL_SAVE_ON_EXIT));
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
                      GTK_SIGNAL_FUNC(save_on_exit_cb), 0);
-  gtk_box_pack_start(GTK_BOX(box), button, 0, 0, GNOME_PAD);
+  gtk_box_pack_start(GTK_BOX(box), button, 0, 0, 0);
+  
+  button = gda_ui_new_check_button_widget(_("Open connection dialog on entry"),
+                                          gnome_config_get_int(FE_CONFIG_GENERAL_CONNECT_ON_ENTRY));
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(connect_on_entry_cb), 0);
+  gtk_box_pack_start(GTK_BOX(box), button, 0, 0, 0);
+  
   gtk_widget_show(box);
   gtk_table_attach(GTK_TABLE(table), box, 0, 2, 0, 1,
                    GTK_FILL | GTK_EXPAND,
                    GTK_FILL | GTK_EXPAND,
                    3, 3);
-  
+
   /* Fonts */
   label = gda_ui_new_label_widget(_("Editor Font"));
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
@@ -281,17 +290,13 @@ general_create_tab (void)
   gtk_table_attach(GTK_TABLE(table), label, 2, 3, 2, 3,
                    GTK_FILL,
                    GTK_FILL, 3, 3);
-  box = gtk_hbox_new(FALSE, 0);
-  entry = gda_ui_new_entry_widget(0, TRUE);
-  gtk_entry_set_text(GTK_ENTRY(entry),
+  entry = gda_ui_new_file_entry_widget("FE_Config_shell_history");
+  gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(entry))),
                      gnome_config_get_string(FE_CONFIG_GENERAL_SHELL));
-  gtk_signal_connect(GTK_OBJECT(entry), "changed",
+  gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(entry))),
+                     "changed",
                      GTK_SIGNAL_FUNC(set_shell_cb), 0);
-  gtk_box_pack_start(GTK_BOX(box), entry, 0, 0, 0);
-  button = fe_new_browse_button_widget(GTK_ENTRY(entry));
-  gtk_box_pack_start(GTK_BOX(box), button, 0, 0, 0);
-  gtk_widget_show(box);
-  gtk_table_attach(GTK_TABLE(table), box, 3, 4, 2, 3,
+  gtk_table_attach(GTK_TABLE(table), entry, 3, 4, 2, 3,
                    GTK_FILL,
                    GTK_FILL, 3, 3);
              
@@ -429,6 +434,19 @@ clear_text_cb (GtkWidget *w, gpointer data)
   gtk_editable_delete_text(GTK_EDITABLE(text), 0, -1);
 }
 
+static void
+connect_on_entry_cb (GtkCheckButton *button, gpointer data)
+{
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
+    {
+      gnome_config_set_int(FE_CONFIG_GENERAL_CONNECT_ON_ENTRY, 1);
+    }
+  else
+    {
+      gnome_config_set_int(FE_CONFIG_GENERAL_CONNECT_ON_ENTRY, 0);
+    }
+}
+
 static void
 current_environ_cb (GtkButton *button, gpointer data)
 {
@@ -531,7 +549,8 @@ save_as_cb (GtkWidget *w, gpointer data)
     }
 }
 
-static void save_config_cb (GtkWidget *w, gpointer data)
+static void
+save_config_cb (GtkWidget *w, gpointer data)
 {
   GtkWidget *text = (GtkWidget *) data;
   gchar *str;
@@ -614,7 +633,7 @@ fe_open_config (GtkWidget *w, gpointer data)
                                   NULL);
       gtk_signal_connect(GTK_OBJECT(l_dialog), "clicked",
                          GTK_SIGNAL_FUNC(dialog_clicked_cb), NULL);
-      l_notebook = fe_new_notebook_widget();
+      l_notebook = gda_ui_new_notebook_widget();
       gtk_widget_show(l_notebook);
       
       /* add tabs */
