@@ -15,20 +15,20 @@
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
- 
-#include "gnome-db-browser.h"
-#include "gnome-db-grid.h"
-#include "gnome-db-list.h"
+
 #include "gda-ui.h"
 
 static void gnome_db_browser_class_init (GnomeDbBrowserClass *);
 static void gnome_db_browser_init       (GnomeDbBrowser *);
 static void gnome_db_browser_destroy    (GnomeDbBrowser *);
 
-static void browse_data_cb (GtkWidget *w, gpointer data);
-static void show_tables_cb (GtkWidget *w, gpointer data);
-static void show_procs_cb  (GtkWidget *w, gpointer data);
-static void show_views_cb  (GtkWidget *w, gpointer data);
+static void browse_data_cb              (GtkWidget *w, gpointer data);
+static void browse_data_loaded_cb       (GnomeDbGrid *grid, gpointer data);
+static void extra_info_cb               (GtkWidget *w, gpointer data);
+static void show_aggrs_cb               (GtkWidget *w, gpointer data);
+static void show_tables_cb              (GtkWidget *w, gpointer data);
+static void show_procs_cb               (GtkWidget *w, gpointer data);
+static void show_views_cb               (GtkWidget *w, gpointer data);
 
 /*
  * GnomeDbBrowser widget signals
@@ -41,142 +41,215 @@ enum
 
 static gint gnome_db_browser_signals[GNOME_DB_BROWSER_LAST_SIGNAL] = { 0, };
 
-/*
- * Private functions
- */
-static void
-show_object_list (GnomeDbBrowser *brw, Gda_Recordset *recset, gint idx)
-{
-  static GtkWidget *pixmap = 0;
-  gchar  bfr[128];
-  gulong position;
-  gint cnt = 0;
-  
-  g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
-  g_return_if_fail(recset != 0);
-
-  position = gda_recordset_move(recset, 1, 0);
-  while (position != GDA_RECORDSET_INVALID_POSITION &&
-         !gda_recordset_eof(recset))
-    {
-      Gda_Field *field = gda_recordset_field_idx(recset, idx);
-      if (field != 0)
-        {
-          gchar *row[2], empty_string[] = "       ";
-          
-          gda_stringify_value(bfr, sizeof(bfr) - 1, field);
-          row[0] = empty_string;
-          row[1] = bfr;
-          gtk_clist_append(GTK_CLIST(brw->object_list), row);
-          
-          /* create shared pixmap */
-          if (!pixmap)
-            {
-              pixmap = gnome_stock_pixmap_widget(brw->object_list, GNOME_STOCK_MENU_BOOK_BLUE);
-              if (!pixmap) return;
-              gtk_widget_show(pixmap);
-            }
-          gtk_clist_set_pixmap(GTK_CLIST(brw->object_list), cnt, 0,
-                               GNOME_PIXMAP(pixmap)->pixmap,
-                               GNOME_PIXMAP(pixmap)->mask);
-        }
-      position = gda_recordset_move(recset, 1, 0);
-      cnt++;
-    }
-  /* free the recordset */
-  gda_recordset_close(recset);
-  gda_recordset_free(recset);
-}
-
 /*
  * Callbacks
  */
 static void
 browse_data_cb (GtkWidget *w, gpointer data)
 {
-  GList *selected;
-  GnomeDbBrowser *brw = GNOME_DB_BROWSER(data);
+  gchar*          name;
+  GnomeDbBrowser* brw = GNOME_DB_BROWSER(data);
   
-  g_return_if_fail(brw != 0);
   g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  g_return_if_fail(GNOME_DB_IS_ICON_LIST(brw->object_list));
   
-  selected = GTK_CLIST(brw->object_list)->selection;
-  if (selected)
+  name = gnome_db_icon_list_get_string(GNOME_DB_ICON_LIST(brw->object_list));
+  if (name)
     {
-      gchar *name = 0;
-      guint row = GPOINTER_TO_UINT(selected->data);
-      gtk_clist_get_text(GTK_CLIST(brw->object_list), row, 1, &name);
-      if (name)
-        {
-          gchar *str = 0;
-          if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->tables_button)) ||
-              gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->views_button)))
-            {
-              str = g_strdup_printf("select * from %s", name);
-            }
+      gchar *str = 0;
+      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->tables_button)) ||
+	  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->views_button)))
+	{
+	  str = g_strdup_printf("select * from %s", name);
+	}
           
-          if (str != 0)
-            {
-              GtkWidget *dialog, *table, *grid;
-              gulong reccount;
-              Gda_Recordset *recset;
+      if (str != 0)
+	{
+	  GtkWidget *dialog, *table, *grid, *label, *entry;
+	  gulong reccount;
+	  Gda_Recordset *recset;
               
-              dialog = gnome_dialog_new(name, GNOME_STOCK_BUTTON_CLOSE, NULL);
-              gtk_widget_set_usize(dialog, 600, 450);
+	  dialog = gnome_dialog_new(name, GNOME_STOCK_BUTTON_CLOSE, NULL);
+	  gtk_widget_set_usize(dialog, 600, 450);
               
-              table = gtk_table_new(3, 2, FALSE);
-              gtk_widget_show(table);
-              gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), table, 1, 1, 0);
+	  table = gtk_table_new(4, 2, FALSE);
+	  gtk_widget_show(table);
+	  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), table, 0, 0, 0);
+
+          label = gda_ui_new_label_widget(_("Object Name"));
+	  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
+                           GTK_FILL,
+                           GTK_FILL,
+			       3, 3);
+          
+          entry = gda_ui_new_entry_widget(0, FALSE);
+	  gtk_entry_set_text(GTK_ENTRY(entry), name);
+	  gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 0, 1,
+			       GTK_FILL,
+                           GTK_FILL,
+			       3, 3);
+
+	  grid = gnome_db_grid_new(0);
+	  gtk_widget_show(grid);
+	  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), grid, 1, 1, 0);
+
+	  label = gda_ui_new_label_widget(_("0 rows"));
+	  gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
+	  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), label, 0, 0, 0);
+	  gtk_signal_connect(GTK_OBJECT(grid), "loaded",
+			     GTK_SIGNAL_FUNC(browse_data_loaded_cb), (gpointer) label);
               
-              grid = gnome_db_grid_new(0);
-              gtk_widget_show(grid);
-              gtk_table_attach(GTK_TABLE(table), grid, 0, 2, 1, 2,
-                               GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-                               GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-                               3, 3);
+	  /* show recordset */
+	  recset = gda_connection_execute(brw->cnc, str, &reccount, 0);
+	  gnome_db_grid_set_recordset(GNOME_DB_GRID(grid), recset);
+	  
+	  gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
               
-              /* show recordset */
-              recset = gda_connection_execute(brw->cnc, str, &reccount, 0);
-              gnome_db_grid_set_recordset(GNOME_DB_GRID(grid), recset);
-              
-              gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
-              
-              g_free((gpointer) str);
-            }
-        }
+	  g_free((gpointer) str);
+	}
     }
 }
 
 static void
-select_object_cb (GtkCList *clist, gint row, gint column, GdkEvent *event, GnomeDbBrowser *brw)
+browse_data_loaded_cb (GnomeDbGrid *grid, gpointer data)
+{
+  gchar *str;
+
+  g_return_if_fail(GNOME_DB_IS_GRID(grid));
+  g_return_if_fail(GTK_IS_LABEL(data));
+
+  str = g_strdup_printf(_("%d rows"), gnome_db_grid_get_row_count(grid));
+  gtk_label_set_text(GTK_LABEL(data), str);
+  g_free((gpointer) str);
+}
+
+static void
+drop_object_cb (GtkWidget *w, gpointer data)
+{
+}
+
+static void
+extra_info_cb (GtkWidget *w, gpointer data)
+{
+  gchar*          name;
+  GnomeDbBrowser* brw = GNOME_DB_BROWSER(data);
+  
+  g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  g_return_if_fail(GNOME_DB_IS_ICON_LIST(brw->object_list));
+  
+  name = gnome_db_icon_list_get_string(GNOME_DB_ICON_LIST(brw->object_list));
+  if (name)
+    {
+      Gda_Recordset* recset;
+
+      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->tables_button)))
+        {
+          recset = gda_connection_open_schema(brw->cnc,
+                                              GDA_Connection_GDCN_SCHEMA_TABLES,
+                                              GDA_Connection_EXTRA_INFO,
+					      "",
+					      GDA_Connection_OBJECT_NAME,
+                                              name,
+                                              GDA_Connection_no_CONSTRAINT);
+        }
+      else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->views_button)))
+        {
+          recset = gda_connection_open_schema(brw->cnc,
+                                              GDA_Connection_GDCN_SCHEMA_VIEWS,
+                                              "",
+                                              GDA_Connection_OBJECT_NAME,
+                                              name,
+                                              GDA_Connection_no_CONSTRAINT);
+        }
+      else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->procs_button)))
+        {
+          recset = gda_connection_open_schema(brw->cnc,
+                                              GDA_Connection_GDCN_SCHEMA_PROCS,
+                                              "",
+                                              GDA_Connection_OBJECT_NAME,
+                                              name,
+                                              GDA_Connection_no_CONSTRAINT);
+        }
+      else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->types_button)))
+        {
+          recset = gda_connection_open_schema(brw->cnc,
+                                              GDA_Connection_GDCN_SCHEMA_PROV_TYPES,
+                                              "",
+                                              GDA_Connection_OBJECT_NAME,
+                                              name,
+                                              GDA_Connection_no_CONSTRAINT);
+        }
+      else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->aggrs_button)))
+        {
+          recset = gda_connection_open_schema(brw->cnc,
+                                              GDA_Connection_GDCN_SCHEMA_AGGREGATES,
+                                              "",
+                                              GDA_Connection_OBJECT_NAME,
+                                              name,
+                                              GDA_Connection_no_CONSTRAINT);
+        }
+      gnome_db_grid_set_recordset(GNOME_DB_GRID(brw->object_description),
+                                  recset);
+    }
+}
+
+static void
+select_object_cb (GnomeDbIconList *icon_list, GnomeDbBrowser *brw)
 {
   gchar *name = 0;
   
+  g_return_if_fail(GNOME_DB_IS_ICON_LIST(icon_list));
   g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
   
-  gtk_clist_get_text(GTK_CLIST(brw->object_list), row, 1, &name);
+  name = gnome_db_icon_list_get_string(icon_list);
   if (name)
     {
-      Gda_Recordset *recset;
+      Gda_Recordset *recset = 0;
       if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->tables_button)) ||
           gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->views_button)))
         {
           recset = gda_connection_open_schema(brw->cnc, 
                                               GDA_Connection_GDCN_SCHEMA_COLS,
-                                              GDA_Connection_TABLE_NAME,
+                                              GDA_Connection_OBJECT_NAME,
                                               name,
                                               GDA_Connection_no_CONSTRAINT);
         }
-      gnome_db_grid_set_recordset(GNOME_DB_GRID(brw->object_description), recset);
-      
-      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->tables_button)) ||
-          gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->views_button)))
+      else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->procs_button)))
         {
-          /* hide all non-table/view specific columns */
-          gnome_db_grid_set_column_visibility(GNOME_DB_GRID(brw->object_description), 0, 0);
-          gnome_db_grid_set_column_visibility(GNOME_DB_GRID(brw->object_description), 1, 0);
-          gnome_db_grid_set_column_visibility(GNOME_DB_GRID(brw->object_description), 2, 0);
+          recset = gda_connection_open_schema(brw->cnc,
+                                              GDA_Connection_GDCN_SCHEMA_PROC_PARAMS,
+                                              GDA_Connection_OBJECT_NAME,
+                                              name,
+                                              GDA_Connection_no_CONSTRAINT);
         }
+      gnome_db_grid_set_recordset(GNOME_DB_GRID(brw->object_description),
+                                  recset);
+    }
+}
+
+static void
+show_aggrs_cb (GtkWidget *w, gpointer data)
+{
+  Gda_Recordset *recset;
+  GnomeDbBrowser *brw = GNOME_DB_BROWSER(data);
+  
+  g_return_if_fail(GTK_IS_RADIO_BUTTON(w));
+  g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  
+  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->aggrs_button)))
+    {
+      return;
+    }
+    
+  /* clean up previous lists */
+  gnome_db_browser_clear(brw);
+  
+  recset = gda_connection_open_schema(brw->cnc,
+                                      GDA_Connection_GDCN_SCHEMA_AGGREGATES,
+                                      GDA_Connection_no_CONSTRAINT, 0);
+  if (recset != 0)
+    {
+      gnome_db_icon_list_set_recordset(GNOME_DB_ICON_LIST(brw->object_list), recset, 0);
     }
 }
 
@@ -186,7 +259,7 @@ show_tables_cb (GtkWidget *w, gpointer data)
   Gda_Recordset *recset;
   GnomeDbBrowser *brw = GNOME_DB_BROWSER(data);
   
-  g_return_if_fail(GTK_IS_TOGGLE_BUTTON(w));
+  g_return_if_fail(GTK_IS_RADIO_BUTTON(w));
   g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
   
   if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->tables_button)))
@@ -197,11 +270,12 @@ show_tables_cb (GtkWidget *w, gpointer data)
   /* clean up previous lists */
   gnome_db_browser_clear(brw);
   
-  recset = gda_connection_open_schema(brw->cnc, GDA_Connection_GDCN_SCHEMA_TABLES,
+  recset = gda_connection_open_schema(brw->cnc,
+                                      GDA_Connection_GDCN_SCHEMA_TABLES,
                                       GDA_Connection_no_CONSTRAINT, 0);
   if (recset != 0)
     {
-      show_object_list(brw, recset, 2);
+      gnome_db_icon_list_set_recordset(GNOME_DB_ICON_LIST(brw->object_list), recset, 0);
     }
 }
 
@@ -211,7 +285,7 @@ show_procs_cb (GtkWidget *w, gpointer data)
   Gda_Recordset *recset;
   GnomeDbBrowser *brw = GNOME_DB_BROWSER(data);
   
-  g_return_if_fail(GTK_IS_TOGGLE_BUTTON(w));
+  g_return_if_fail(GTK_IS_RADIO_BUTTON(w));
   g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
   
   if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->procs_button)))
@@ -222,11 +296,38 @@ show_procs_cb (GtkWidget *w, gpointer data)
   /* clean up previous lists */
   gnome_db_browser_clear(brw);
   
-  recset = gda_connection_open_schema(brw->cnc, GDA_Connection_GDCN_SCHEMA_PROCS,
+  recset = gda_connection_open_schema(brw->cnc,
+                                      GDA_Connection_GDCN_SCHEMA_PROCS,
                                       GDA_Connection_no_CONSTRAINT, 0);
   if (recset != 0)
     {
-      show_object_list(brw, recset, 2);
+      gnome_db_icon_list_set_recordset(GNOME_DB_ICON_LIST(brw->object_list), recset, 0);
+    }
+}
+
+static void
+show_types_cb (GtkWidget *w, gpointer data)
+{
+  Gda_Recordset *recset;
+  GnomeDbBrowser *brw = GNOME_DB_BROWSER(data);
+  
+  g_return_if_fail(GTK_IS_RADIO_BUTTON(w));
+  g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  
+  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->types_button)))
+    {
+      return;
+    }
+    
+  /* clean up previous lists */
+  gnome_db_browser_clear(brw);
+  
+  recset = gda_connection_open_schema(brw->cnc,
+                                      GDA_Connection_GDCN_SCHEMA_PROV_TYPES,
+                                      GDA_Connection_no_CONSTRAINT, 0);
+  if (recset != 0)
+    {
+      gnome_db_icon_list_set_recordset(GNOME_DB_ICON_LIST(brw->object_list), recset, 0);
     }
 }
 
@@ -236,7 +337,7 @@ show_views_cb (GtkWidget *w, gpointer data)
   Gda_Recordset *recset;
   GnomeDbBrowser *brw = GNOME_DB_BROWSER(data);
   
-  g_return_if_fail(GTK_IS_TOGGLE_BUTTON(w));
+  g_return_if_fail(GTK_IS_RADIO_BUTTON(w));
   g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
   
   if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->views_button)))
@@ -247,12 +348,13 @@ show_views_cb (GtkWidget *w, gpointer data)
   /* clean up previous lists */
   gnome_db_browser_clear(brw);
   
-  recset = gda_connection_open_schema(brw->cnc, GDA_Connection_GDCN_SCHEMA_VIEWS,
+  recset = gda_connection_open_schema(brw->cnc,
+                                      GDA_Connection_GDCN_SCHEMA_VIEWS,
                                       GDA_Connection_no_CONSTRAINT, 0);
                                       
   if (recset != 0)
     {
-      show_object_list(brw, recset, 0);
+      gnome_db_icon_list_set_recordset(GNOME_DB_ICON_LIST(brw->object_list), recset, 0);
     }
 }
 
@@ -282,23 +384,16 @@ gnome_db_browser_init (GnomeDbBrowser *brw)
   GtkWidget *table;
   GtkWidget *box;
   GtkWidget *button;
-  GtkWidget *scroll;
   
   table = gtk_table_new(5, 8, FALSE);
   gtk_box_pack_start(GTK_BOX(brw), table, 1, 1, 0);
   gtk_widget_show(table);
   
-  scroll = gtk_scrolled_window_new(0, 0);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
-                                 GTK_POLICY_AUTOMATIC,
-                                 GTK_POLICY_AUTOMATIC);
-  brw->object_list = gda_ui_new_clist_widget(0, 2);
+  brw->object_list = gnome_db_icon_list_new(0, 0);
   gtk_signal_connect(GTK_OBJECT(brw->object_list), "select_row",
                      GTK_SIGNAL_FUNC(select_object_cb), (gpointer) brw);
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll), brw->object_list);
   gtk_widget_show(brw->object_list);
-  gtk_widget_show(scroll);
-  gtk_table_attach(GTK_TABLE(table), scroll, 0, 6, 0, 1,
+  gtk_table_attach(GTK_TABLE(table), brw->object_list, 0, 6, 0, 1,
                    GTK_FILL | GTK_EXPAND | GTK_SHRINK,
                    GTK_FILL | GTK_EXPAND | GTK_SHRINK,
                    3, 3);
@@ -334,19 +429,35 @@ gnome_db_browser_init (GnomeDbBrowser *brw)
                      GTK_SIGNAL_FUNC(show_procs_cb), (gpointer) brw);
   gtk_widget_show(brw->procs_button);
   gtk_box_pack_start(GTK_BOX(box), brw->procs_button, 0, 0, 0);
+
+  brw->types_button = gtk_radio_button_new_with_label(
+				gtk_radio_button_group(GTK_RADIO_BUTTON(brw->tables_button)),
+				_("Types"));
+  gtk_signal_connect(GTK_OBJECT(brw->types_button), "toggled",
+                     GTK_SIGNAL_FUNC(show_types_cb), (gpointer) brw);
+  gtk_widget_show(brw->types_button);
+  gtk_box_pack_start(GTK_BOX(box), brw->types_button, 0, 0, 0);
+
+  brw->aggrs_button = gtk_radio_button_new_with_label(
+                                gtk_radio_button_group(GTK_RADIO_BUTTON(brw->tables_button)),
+                                _("Aggregates"));
+  gtk_signal_connect(GTK_OBJECT(brw->aggrs_button), "toggled",
+                     GTK_SIGNAL_FUNC(show_aggrs_cb), (gpointer) brw);
+  gtk_widget_show(brw->aggrs_button);
+  gtk_box_pack_start(GTK_BOX(box), brw->aggrs_button, 0, 0, 0);
   
   gtk_table_attach(GTK_TABLE(table), box, 6, 7, 0, 1,
                    GTK_SHRINK, 
-                   0,
+                   GTK_FILL,
                    3, 3);
   gtk_widget_show(box);
   
-  /* create left button bar */
+  /* create right button bar */
   box = gtk_vbox_new(FALSE, 0);
   gtk_widget_show(box);
   gtk_table_attach(GTK_TABLE(table), box, 7, 8, 0, 1,
                    GTK_SHRINK,
-                   0,
+                   GTK_FILL,
                    3, 3);
                    
   button = gnome_pixmap_button(gnome_stock_new_with_icon(GNOME_STOCK_MENU_PROP),
@@ -360,10 +471,16 @@ gnome_db_browser_init (GnomeDbBrowser *brw)
   button = gnome_pixmap_button(gnome_stock_new_with_icon(GNOME_STOCK_MENU_TRASH),
                                _("Remove"));
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-  //gtk_signal_connect(GTK_OBJECT(button), "clicked",
-  //                   GTK_SIGNAL_FUNC(browse_data_cb), (gpointer) brw);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(drop_object_cb), (gpointer) brw);
   gtk_widget_show(button);
   gtk_box_pack_start(GTK_BOX(box), button, 0, 0, 0);
+  
+  button = gda_ui_new_button_widget_with_pixmap(_("Extra Info"), GNOME_STOCK_MENU_PROP);
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(extra_info_cb), (gpointer) brw);
+  gtk_box_pack_start(GTK_BOX(box), button, 0, 0, 0);
 }
 
 guint
@@ -403,7 +520,8 @@ gnome_db_browser_new (Gda_Connection *cnc)
   GtkWidget *brw;
 
   brw = gtk_type_new(gnome_db_browser_get_type());
-  GNOME_DB_BROWSER(brw)->cnc = cnc;
+  if (IS_GDA_CONNECTION(cnc))
+    gnome_db_browser_set_connection(GNOME_DB_BROWSER(brw), cnc);
   return (brw);
 }
 
@@ -417,7 +535,8 @@ static void
 gnome_db_browser_destroy (GnomeDbBrowser *brw)
 {
   g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
-  
+
+  gtk_widget_destroy(brw->object_list);  
   gtk_widget_destroy(brw->object_description);
 }
 
@@ -456,9 +575,13 @@ gnome_db_browser_set_connection (GnomeDbBrowser *brw, Gda_Connection *cnc)
   /* update object list */
   gnome_db_browser_clear(brw);
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->tables_button)))
-    show_tables_cb(brw, (gpointer) brw);
+    show_tables_cb(brw->tables_button, (gpointer) brw);
   else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->views_button)))
-    show_views_cb(brw, (gpointer) brw);
+    show_views_cb(brw->views_button, (gpointer) brw);
+  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->procs_button)))
+    show_procs_cb(brw->procs_button, (gpointer) brw);
+  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->types_button)))
+    show_types_cb(brw->types_button, (gpointer) brw);
 
   gtk_signal_emit(GTK_OBJECT(brw), 
                   gnome_db_browser_signals[GNOME_DB_BROWSER_CONNECTION_CHANGED]);
@@ -475,7 +598,7 @@ gnome_db_browser_clear (GnomeDbBrowser *brw)
 {
   g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
   
-  gda_ui_clear_clist(GTK_CLIST(brw->object_list));
+  gnome_db_icon_list_clear(GNOME_DB_ICON_LIST(brw->object_list));
   gnome_db_grid_set_recordset(GNOME_DB_GRID(brw->object_description), 0);
 }
 
@@ -489,6 +612,61 @@ gnome_db_browser_clear (GnomeDbBrowser *brw)
 void
 gnome_db_browser_refresh (GnomeDbBrowser *brw)
 {
-  g_return_if_fail(brw != NULL);
+  g_return_if_fail(brw != 0);
   g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  
+  gnome_db_browser_clear(brw);
+  
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->tables_button)))
+    show_tables_cb(GTK_RADIO_BUTTON(brw->tables_button), brw);
+  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->views_button)))
+    show_views_cb(GTK_RADIO_BUTTON(brw->views_button), brw);
+  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->procs_button)))
+    show_procs_cb(GTK_RADIO_BUTTON(brw->procs_button), brw);
+  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(brw->types_button)))
+    show_types_cb(GTK_RADIO_BUTTON(brw->types_button), brw);
+}
+
+/**
+ * gnome_db_browser_show_procs
+ * @brw: the GnomeDbBrowser widget
+ */
+void
+gnome_db_browser_show_procs (GnomeDbBrowser *brw)
+{
+  g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(brw->procs_button), TRUE);
+}
+
+/**
+ * gnome_db_browser_show_tables
+ * @brw: the GnomeDbBrowser widget
+ */
+void
+gnome_db_browser_show_tables (GnomeDbBrowser *brw)
+{
+  g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(brw->tables_button), TRUE);
+}
+
+/**
+ * gnome_db_browser_show_types
+ * @brw: the GnomeDbBrowser widget
+ */
+void
+gnome_db_browser_show_types (GnomeDbBrowser *brw)
+{
+  g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(brw->types_button), TRUE);
+}
+
+/**
+ * gnome_db_browser_show_views
+ * @brw: the GnomeDbBrowser widget
+ */
+void
+gnome_db_browser_show_views (GnomeDbBrowser *brw)
+{
+  g_return_if_fail(GNOME_DB_IS_BROWSER(brw));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(brw->views_button), TRUE);
 }
