@@ -33,6 +33,7 @@ enum
   GNOME_DB_GRID_LOADED,
   GNOME_DB_GRID_SELECT_ROW,
   GNOME_DB_GRID_UNSELECT_ROW,
+  GNOME_DB_GRID_ADD_ROW,
   GNOME_DB_GRID_LAST_SIGNAL
 };
 
@@ -212,13 +213,17 @@ timeout_callback (GnomeDbGrid *grid)
     {
       gchar *str;
       GPtrArray *row = g_ptr_array_new();
+
+      gtk_clist_freeze(GTK_CLIST(grid->grid));
       for (cnt = 0; cnt < gda_recordset_rowsize(grid->recset); cnt++)
         {
           Gda_Field *field = gda_recordset_field_idx(grid->recset, cnt);
           g_ptr_array_add(row, (gpointer) gda_stringify_value(0, 0, field));
         }
-      gtk_clist_freeze(GTK_CLIST(grid->grid));
       gtk_clist_append(GTK_CLIST(grid->grid), (gchar **) row->pdata);
+      gtk_signal_emit(GTK_OBJECT(grid),
+                      gnome_db_grid_signals[GNOME_DB_GRID_ADD_ROW],
+                      grid->total_rows);
       gtk_clist_thaw(GTK_CLIST(grid->grid));
       g_ptr_array_free(row, TRUE);
       
@@ -271,6 +276,32 @@ show_recordset (GnomeDbGrid *grid, Gda_Recordset *recset)
           }
         grid->grid = create_grid(total_fields, (gchar **) columns->pdata, 
                                  grid);
+
+        /* column justification */
+        for (cnt = 0; cnt < total_fields; cnt++)
+          {
+            switch (gda_field_type(gda_recordset_field_idx(recset, cnt)))
+              {
+                case GDA_TypeBigint :
+                case GDA_TypeCurrency :
+                case GDA_TypeDecimal :
+                case GDA_TypeDouble :
+                case GDA_TypeInteger :
+                case GDA_TypeNumeric :
+                case GDA_TypeSingle :
+                case GDA_TypeSmallint :
+                case GDA_TypeTinyint :
+                case GDA_TypeUBigint :
+                case GDA_TypeUSmallint :
+                  gtk_clist_set_column_justification(GTK_CLIST(grid->grid), cnt,
+                                                     GTK_JUSTIFY_RIGHT);
+                  break;
+                default :
+                  gtk_clist_set_column_justification(GTK_CLIST(grid->grid), cnt,
+                                                     GTK_JUSTIFY_LEFT);
+                  break;
+              }
+          }
       }
     g_ptr_array_free(columns, TRUE);
   }
@@ -290,25 +321,30 @@ gnome_db_grid_class_init (GnomeDbGridClass *klass)
   GtkObjectClass *object_class = (GtkObjectClass *) klass;
 
   gnome_db_grid_signals[GNOME_DB_GRID_ERROR] = 
-            gtk_signal_new ("error", GTK_RUN_FIRST, object_class->type,
-                            GTK_SIGNAL_OFFSET(GnomeDbGridClass, error),
-                            gtk_signal_default_marshaller, 
-                            GTK_TYPE_NONE, 0);
-  gnome_db_grid_signals[GNOME_DB_GRID_LOADED] =
-            gtk_signal_new("loaded", GTK_RUN_LAST, object_class->type,
-                           GTK_SIGNAL_OFFSET(GnomeDbGridClass, loaded),
-                           gtk_signal_default_marshaller,
+           gtk_signal_new ("error", GTK_RUN_FIRST, object_class->type,
+                           GTK_SIGNAL_OFFSET(GnomeDbGridClass, error),
+                           gtk_signal_default_marshaller, 
                            GTK_TYPE_NONE, 0);
+  gnome_db_grid_signals[GNOME_DB_GRID_LOADED] =
+           gtk_signal_new("loaded", GTK_RUN_LAST, object_class->type,
+                          GTK_SIGNAL_OFFSET(GnomeDbGridClass, loaded),
+                          gtk_signal_default_marshaller,
+                          GTK_TYPE_NONE, 0);
   gnome_db_grid_signals[GNOME_DB_GRID_SELECT_ROW] =
-            gtk_signal_new("select_row", GTK_RUN_LAST, object_class->type,
-                           GTK_SIGNAL_OFFSET(GnomeDbGridClass, select_row),
-                           gtk_signal_default_marshaller,
-                           GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+           gtk_signal_new("select_row", GTK_RUN_LAST, object_class->type,
+                          GTK_SIGNAL_OFFSET(GnomeDbGridClass, select_row),
+                          gtk_signal_default_marshaller,
+                          GTK_TYPE_NONE, 1, GTK_TYPE_INT);
   gnome_db_grid_signals[GNOME_DB_GRID_UNSELECT_ROW] =
-            gtk_signal_new("unselect_row", GTK_RUN_LAST, object_class->type,
-                           GTK_SIGNAL_OFFSET(GnomeDbGridClass, unselect_row),
-                           gtk_signal_default_marshaller,
-                           GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+           gtk_signal_new("unselect_row", GTK_RUN_LAST, object_class->type,
+                          GTK_SIGNAL_OFFSET(GnomeDbGridClass, unselect_row),
+                          gtk_signal_default_marshaller,
+                          GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+  gnome_db_grid_signals[GNOME_DB_GRID_ADD_ROW] =
+           gtk_signal_new("add_row", GTK_RUN_LAST, object_class->type,
+                          GTK_SIGNAL_OFFSET(GnomeDbGridClass, add_row),
+                          gtk_signal_default_marshaller,
+                          GTK_TYPE_NONE, 1, GTK_TYPE_INT);
   gtk_object_class_add_signals(object_class, gnome_db_grid_signals,
                                GNOME_DB_GRID_LAST_SIGNAL);
 
@@ -316,6 +352,7 @@ gnome_db_grid_class_init (GnomeDbGridClass *klass)
   klass->loaded = 0;
   klass->select_row = 0;
   klass->unselect_row = 0;
+  klass->add_row = 0;
   object_class->destroy = gnome_db_grid_destroy;
 }
 
@@ -665,6 +702,13 @@ gnome_db_grid_set_recordset (GnomeDbGrid *grid, Gda_Recordset *recset)
   g_return_if_fail(grid != NULL);
   g_return_if_fail(GNOME_DB_IS_GRID(grid));
 
+  /* stop timeout if active */
+  if (grid->timeout_handle != -1)
+    {
+      gtk_timeout_remove(grid->timeout_handle);
+      grid->timeout_handle = -1;
+    }
+  
   if (recset == grid->recset && recset != NULL)
     gnome_db_grid_refresh(grid);
   else
