@@ -17,13 +17,7 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <gnome.h>
-
-#include <gda.h>
-#include "gda-command.h"
-#include "gda-recordset.h"
-#include "gda-connection.h"
-#include "gda-error.h"
+#include <gda-connection.h>
 
 enum
 {
@@ -52,8 +46,6 @@ release_connection_object (Gda_Command* cmd, Gda_Connection* cnc)
  * this function will change as soon as the client side
  * error objects are here
  */
-
-
 static gint
 Exception (CORBA_Environment *ev)
 {
@@ -79,13 +71,13 @@ gda_command_get_type (void)
     {
       GtkTypeInfo gda_command_info =
       {
-	"GdaCommand",
-	sizeof (Gda_Command),
-	sizeof (Gda_CommandClass),
-	(GtkClassInitFunc) gda_command_class_init,
-	(GtkObjectInitFunc) gda_command_init,
-	(GtkArgSetFunc)NULL,
-	(GtkArgSetFunc)NULL,
+        "GdaCommand",
+        sizeof (Gda_Command),
+        sizeof (Gda_CommandClass),
+        (GtkClassInitFunc) gda_command_class_init,
+        (GtkObjectInitFunc) gda_command_init,
+        (GtkArgSetFunc)NULL,
+        (GtkArgSetFunc)NULL,
       };
       gda_command_type = gtk_type_unique(gtk_object_get_type(), &gda_command_info);
     }
@@ -103,7 +95,7 @@ gda_command_class_init (Gda_CommandClass* klass)
 static void
 gda_command_init (Gda_Command* cmd)
 {
-  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+  g_return_if_fail(IS_GDA_COMMAND(cmd));
 }
 
 /**
@@ -113,8 +105,7 @@ gda_command_init (Gda_Command* cmd)
  *
  * Returns: a pointer to the command object
  */
-
-Gda_Command*
+Gda_Command *
 gda_command_new (void)
 {
   Gda_Command* cmd;
@@ -132,7 +123,6 @@ gda_command_new (void)
  * cuts the association with its connection object.
  *
  */
-
 void
 gda_command_free (Gda_Command* cmd)
 {
@@ -145,10 +135,9 @@ gda_command_free (Gda_Command* cmd)
   else
     {
       if (cmd->connection && !cmd->connection->commands)
-	g_error("gda_command_free: connection object has no command list");
+        g_error("gda_command_free: connection object has no command list");
     }
-  if (cmd->text)
-    g_free(cmd->text);
+  if (cmd->text) g_free(cmd->text);
 
   if (cmd->command != CORBA_OBJECT_NIL)
     {
@@ -212,7 +201,7 @@ gda_command_set_connection (Gda_Command* cmd, Gda_Connection* cnc)
  *
  * Returns: a pointer to the #gda_Connection object
  */
-Gda_Connection*
+Gda_Connection *
 gda_command_get_connection (Gda_Command* cmd)
 {
   g_return_val_if_fail(IS_GDA_COMMAND(cmd), 0);
@@ -231,7 +220,6 @@ gda_command_get_connection (Gda_Command* cmd)
  * function is called.
  *
  */
-
 void
 gda_command_set_text (Gda_Command* cmd, gchar* text)
 {
@@ -264,8 +252,7 @@ gda_command_set_text (Gda_Command* cmd, gchar* text)
  *
  * Returns: a reference to the command string.
  */
-
-gchar*
+gchar *
 gda_command_get_text (Gda_Command* cmd)
 {
   gchar* txt;
@@ -361,33 +348,32 @@ __gda_command_get_params (Gda_Command* cmd)
       ptr = cmd->parameters;
       idx = 0;
       while(ptr)
-	{
-	  GDA_Value* value;
-	  Parameter* parameter;
+        {
+          GDA_Value* value;
+          Parameter* parameter;
 	  
-	  parameter = ptr->data;
-	  corba_parameter = &(corba_parameters->_buffer[idx]);
+          parameter = ptr->data;
+          corba_parameter = &(corba_parameters->_buffer[idx]);
 
-	  corba_parameter->dir = parameter->inout;
-	  if (parameter->name)
-	    {
-	      corba_parameter->name = CORBA_string_dup(parameter->name);
-	    }
-	  else
-	    corba_parameter->name = 0;
+          corba_parameter->dir = parameter->inout;
+          if (parameter->name)
+            {
+              corba_parameter->name = CORBA_string_dup(parameter->name);
+            }
+          else corba_parameter->name = 0;
 
-	  value = parameter->value;
-	  if (!(corba_parameter->value._d = value ? 0 : 1))
-	    {
-	      corba_parameter->value._u.v = *((*parameter).value);
-	    }
-	  else
-	    {
-	      fprintf(stderr,"Got NULL param value\n");
-	    }
-	  ptr = g_list_next(ptr);
-	  idx++;
-	}
+          value = parameter->value;
+          if (!(corba_parameter->value._d = value ? 0 : 1))
+            {
+              corba_parameter->value._u.v = *((*parameter).value);
+            }
+          else
+            {
+              fprintf(stderr,"Got NULL param value\n");
+            }
+          ptr = g_list_next(ptr);
+          idx++;
+        }
     }
   return corba_parameters;
 }
@@ -406,8 +392,7 @@ __gda_command_get_params (Gda_Command* cmd)
  *
  * Returns: a pointer to a recordset or a NULL pointer if there was an error.
  */
-
-Gda_Recordset*
+Gda_Recordset *
 gda_command_execute (Gda_Command* cmd, gulong* reccount, gulong flags)
 {
   gint rc;
@@ -424,7 +409,7 @@ gda_command_execute (Gda_Command* cmd, gulong* reccount, gulong flags)
       gda_recordset_free(rs);
       return 0;
     }
-  *reccount = 0;
+  *reccount = rs->affected_rows;
   return rs;
 }
 
@@ -462,7 +447,7 @@ void
 gda_command_create_parameter (Gda_Command* cmd,
                               gchar* name,
                               GDA_ParameterDirection inout,
-			      GDA_Value* value)
+                              GDA_Value* value)
 {
   Parameter* param;
   
