@@ -22,14 +22,12 @@
 
 #include <glib.h>
 #include <orb/orbit.h>
+
 #include <gda.h>
-#include "gda-connection.h"
-#include "gda-recordset.h"
 
 /* The command object. Holds and executes an SQL query or other,
  * datasource specific operations.
  */
-
 typedef struct _Gda_Command      Gda_Command;
 typedef struct _Gda_CommandClass Gda_CommandClass;
 
@@ -39,7 +37,6 @@ typedef struct _Gda_CommandClass Gda_CommandClass;
 #define IS_GDA_COMMAND(obj)         GTK_CHECK_TYPE(obj, GDA_TYPE_COMMAND)
 #define IS_GDA_COMMAND_CLASS(klass) (GTK_CHECK_CLASS_TYPE((klass), GDA_TYPE_COMMAND))
 
-
 struct _Gda_Command
 {
   GtkObject       object;
@@ -68,8 +65,10 @@ void            gda_command_set_text         (Gda_Command* cmd, gchar* text);
 gulong          gda_command_get_cmd_type     (Gda_Command* cmd);
 void            gda_command_set_cmd_type     (Gda_Command* cmd, gulong flags);
 Gda_Recordset*  gda_command_execute          (Gda_Command* cmd, gulong* reccount, gulong flags);
-void            gda_command_create_parameter (Gda_Command* cmd, gchar* name,
-					      GDA_ParameterDirection inout, GDA_Value* value);
+void            gda_command_create_parameter (Gda_Command* cmd,
+                                              gchar* name,
+                                              GDA_ParameterDirection inout,
+                                              GDA_Value* value);
 glong           gda_command_get_timeout      (Gda_Command* cmd);
 void            gda_command_set_timeout      (Gda_Command* cmd, glong timeout);
 
