@@ -22,10 +22,9 @@
 #include <libgnorba/gnorba.h>
 #include <gtk/gtkmarshal.h>
 
-#include "gda-connection.h"
-#include "gda-server.h"
-#include "gda-error.h"
-#include "gda-command.h"
+#include <gda-connection.h>
+#include <gda-recordset.h>
+#include <gda-command.h>
 
 static GHashTable* factories = 0;
 
@@ -45,8 +44,6 @@ static void    gda_connection_class_init    (Gda_ConnectionClass* klass);
 static void    gda_connection_init          (Gda_Connection* cnc);
 static void    gda_connection_real_error    (Gda_Connection* cnc, GList*);
 static void    gda_connection_real_warning  (Gda_Connection* cnc, GList*);
-static void    gda_connection_real_open     (Gda_Connection* cnc);
-static void    gda_connection_real_close    (Gda_Connection* cnc);
 static int     get_corba_connection         (Gda_Connection* cnc);
 
 static void
@@ -62,84 +59,6 @@ gda_connection_real_warning (Gda_Connection* cnc, GList* warnings)
   g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
 }
 
-/**
- * gda_connection_real_open:
- * @cnc: the connection object which describes the server and
- *        the database name to which the connection shuld be opened
- * @dsn: The DSN of the database. Can be NULL.
- * @user: The username for authentication to the database. Can be NULL.
- * @pwd: The password for authentication to the database. Can be NULL.
- * 
- * The function activates the correct CORBA server (defined with he
- * #gda_connection_set_provider() function. Then it
- * tries to open the database using the DSN or the default database
- * as a data source. If @user or @pwd is not NULL, it will overwrite the
- * appropriate entry in the DSN passed as @par2. Entries in the DSN have the form
- * <key> = <value> seperated from the database name . Currently the DSN is not
- * parsed.
- *
- * Returns: 0 on success, -1 on error
- */
-static void
-gda_connection_real_open (Gda_Connection* cnc)
-{
-  CORBA_Environment     ev;
-  gint                  rc;
-  
-  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
-  
-  if (get_corba_connection(cnc) < 0)
-      return;
-
-  CORBA_exception_init(&ev);
-
-  rc = GDA_Connection_open(cnc->connection, cnc->database, cnc->user, cnc->passwd, &ev);
-  g_print("GDA_Connection_open returns %d\n", rc);
-  if (gda_connection_corba_exception(cnc, &ev) < 0)
-    {
-      CORBA_SystemException* sysexc = CORBA_exception_value(&ev);
-      if (sysexc->completed != CORBA_COMPLETED_NO)
-	{
-	  GDA_Connection_close(cnc->connection, &ev);
-	  CORBA_Object_release(cnc->connection, &ev);
-	}
-      cnc->connection = CORBA_OBJECT_NIL;
-      return ;
-    }
-  
-  cnc->is_open = 1;
-  return;
-}
-
-/**
- * gda_connection_real_close:
- * @cnc: The connection object which should be closed
- *
- * Closes the connection object and all associated #Gda_Recordset objects.
- * #Gda_Command objects are not closed, but can't be used anymore. Transactions
- * pending on this objects are aborted and an error is returned.
- *
- * Returns: 0 on success, -1 on error
- */
-static void
-gda_connection_real_close (Gda_Connection* cnc)
-{
-  CORBA_Environment     ev;
-  
-  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
-
-  g_return_if_fail(cnc != NULL);
-  g_return_if_fail(cnc->connection != NULL);
-
-  CORBA_exception_init(&ev);
-  GDA_Connection_close(cnc->connection, &ev);
-  gda_connection_corba_exception(cnc, &ev);
-  CORBA_Object_release(cnc->connection, &ev);
-  cnc->is_open = 0;
-  cnc->connection = CORBA_OBJECT_NIL;
-  return;
-}
-
 guint
 gda_connection_get_type (void)
 {
@@ -149,13 +68,13 @@ gda_connection_get_type (void)
     {
       GtkTypeInfo gda_connection_info =
       {
-	"GdaConnection",
-	sizeof (Gda_Connection),
-	sizeof (Gda_ConnectionClass),
-	(GtkClassInitFunc)gda_connection_class_init,
-	(GtkObjectInitFunc)gda_connection_init,
-	(GtkArgSetFunc)NULL,
-	(GtkArgSetFunc)NULL,
+        "GdaConnection",
+        sizeof (Gda_Connection),
+        sizeof (Gda_ConnectionClass),
+        (GtkClassInitFunc)gda_connection_class_init,
+        (GtkObjectInitFunc)gda_connection_init,
+        (GtkArgSetFunc)NULL,
+        (GtkArgSetFunc)NULL,
       };
       gda_connection_type = gtk_type_unique(gtk_object_get_type(),
                                             &gda_connection_info);
@@ -199,9 +118,8 @@ gda_connection_class_init (Gda_ConnectionClass* klass)
   gtk_object_class_add_signals(object_class, gda_connection_signals, LAST_SIGNAL);
   klass->error   = gda_connection_real_error;
   klass->warning = gda_connection_real_warning;
-  klass->open    = gda_connection_real_open;
-  klass->close   = gda_connection_real_close;
-  
+  klass->close   = NULL;
+  klass->open    = NULL;
 }
 
 static void
@@ -210,8 +128,6 @@ gda_connection_init (Gda_Connection* cnc)
   g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
 }
 
-
-  
 static guint
 hash_id_hash (gconstpointer data)
 {
@@ -257,57 +173,58 @@ gda_connection_corba_exception (Gda_Connection* cnc, CORBA_Environment* ev)
       return 0;
     case CORBA_SYSTEM_EXCEPTION:
       {
-	CORBA_SystemException* sysexc;
+        CORBA_SystemException* sysexc;
 
-	sysexc = CORBA_exception_value(ev);
-	error = gda_error_new();
-	error->source = g_strdup("[CORBA System Exception]");
-	switch(sysexc->minor)
-	  {
-	  case ex_CORBA_COMM_FAILURE:
-	    error->description = g_strdup_printf(_("%s: The server didn't respond."), CORBA_exception_id(ev));
-	    break;
-	  default:
-	    error->description = g_strdup(_("%s: An Error occured in the CORBA system."));
-	    break;
-	  }
-	gda_connection_add_single_error(cnc, error);
-	return -1;
+        sysexc = CORBA_exception_value(ev);
+        error = gda_error_new();
+        error->source = g_strdup("[CORBA System Exception]");
+        switch(sysexc->minor)
+          {
+          case ex_CORBA_COMM_FAILURE:
+            error->description = g_strdup_printf(_("%s: The server didn't respond."),
+                                                 CORBA_exception_id(ev));
+            break;
+          default:
+            error->description = g_strdup(_("%s: An Error occured in the CORBA system."));
+            break;
+          }
+        gda_connection_add_single_error(cnc, error);
+        return -1;
       }
     case CORBA_USER_EXCEPTION:
       if (strcmp(CORBA_exception_id(ev), ex_GDA_NotSupported) == 0)
-	{
-	  GDA_NotSupported* not_supported_error;
+        {
+          GDA_NotSupported* not_supported_error;
 
-	  not_supported_error = (GDA_NotSupported*)ev->_params;
-	  error = gda_error_new();
-	  error->source = g_strdup("[CORBA User Exception]");
-	  error->description = g_strdup(not_supported_error->errormsg);
-	  gda_connection_add_single_error(cnc, error);
-	  return -1;
-	}
+          not_supported_error = (GDA_NotSupported*)ev->_params;
+          error = gda_error_new();
+          error->source = g_strdup("[CORBA User Exception]");
+          error->description = g_strdup(not_supported_error->errormsg);
+          gda_connection_add_single_error(cnc, error);
+          return -1;
+        }
       if (strcmp(CORBA_exception_id(ev), ex_GDA_DriverError) == 0)
-	{
-	  GDA_ErrorSeq  error_sequence = ((GDA_DriverError*)ev->_params)->errors;
-	  gint          idx;
+        {
+          GDA_ErrorSeq  error_sequence = ((GDA_DriverError*)ev->_params)->errors;
+          gint          idx;
 
-	  for (idx = 0; idx < error_sequence._length; idx++)
-	    {
-	      GDA_Error* gda_error = &error_sequence._buffer[idx];
+          for (idx = 0; idx < error_sequence._length; idx++)
+            {
+              GDA_Error* gda_error = &error_sequence._buffer[idx];
 
-	      error = gda_error_new();
-	      if (gda_error->source)
-		error->source = g_strdup(gda_error->source);
-	      error->number = gda_error->number;
-	      if (gda_error->sqlstate)
-		error->sqlstate = g_strdup(gda_error->sqlstate);
-	      if (gda_error->nativeMsg)
-		error->nativeMsg = g_strdup(gda_error->nativeMsg);
-	      if (gda_error->description)
-		error->description = g_strdup(gda_error->description);
-	      gda_connection_add_single_error(cnc, error);
-	    }
-	}
+              error = gda_error_new();
+              if (gda_error->source)
+                error->source = g_strdup(gda_error->source);
+              error->number = gda_error->number;
+              if (gda_error->sqlstate)
+                error->sqlstate = g_strdup(gda_error->sqlstate);
+              if (gda_error->nativeMsg)
+                error->nativeMsg = g_strdup(gda_error->nativeMsg);
+              if (gda_error->description)
+                error->description = g_strdup(gda_error->description);
+              gda_connection_add_single_error(cnc, error);
+            }
+        }
       return -1;
     default:
       g_error("Unknown CORBA exception for connection");
@@ -414,8 +331,7 @@ gda_connection_set_provider (Gda_Connection* cnc, gchar* name)
   g_return_if_fail(IS_GDA_CONNECTION(cnc));
   g_return_if_fail(name != 0);
 
-  if (cnc->provider)
-    g_free(cnc->provider);
+  if (cnc->provider) g_free(cnc->provider);
   cnc->provider = g_strdup(name);
 }
 
@@ -472,12 +388,39 @@ gda_connection_list_datasources (Gda_Connection* cnc)
   return retval;
 }
 
+/**
+ * gda_connection_supports
+ * @cnc: the connection object
+ * @feature: feature to be tested
+ *
+ * Return information on features supported by the underlying database
+ * system.
+ *
+ * Returns: whether the requested feature is supported or not
+ */
+gboolean
+gda_connection_supports (Gda_Connection *cnc, GDA_Connection_Feature feature)
+{
+  CORBA_Environment ev;
+  gboolean          rc;
+
+  g_return_val_if_fail(IS_GDA_CONNECTION(cnc), FALSE);
+
+  CORBA_exception_init(&ev);
+  rc = GDA_Connection_supports(cnc->connection, feature, &ev);
+  if (gda_connection_corba_exception(cnc, &ev) < 0)
+    return FALSE;
+
+  return rc;
+}
+
 /**
  * gda_connection_set_default_db:
  * @cnc: the connection object
  * @dsn: the DSN of the default database
  *
- * Registers the DSN as the name of the default DB to access when the connection is opened
+ * Registers the DSN as the name of the default DB to access when the
+ * connection is opened
  *
  */
 void
@@ -513,9 +456,9 @@ gda_connection_set_default_db (Gda_Connection* cnc, gchar* dsn)
 gint
 gda_connection_open (Gda_Connection* cnc, gchar* dsn, gchar* user, gchar* pwd)
 {
-  gchar* db_to_use;
-  gchar* user_to_use;
-  gchar* pwd_to_use;
+  gchar*            db_to_use;
+  gint              rc;
+  CORBA_Environment ev;
   
   g_return_val_if_fail(IS_GDA_CONNECTION(cnc), -1);
   g_return_val_if_fail(cnc->connection == CORBA_OBJECT_NIL, -1);
@@ -525,41 +468,52 @@ gda_connection_open (Gda_Connection* cnc, gchar* dsn, gchar* user, gchar* pwd)
 
   if (!dsn)
     db_to_use = cnc->default_db;
-  else
+  else db_to_use = dsn;
+  if (!db_to_use)
     {
-      size_t pos = strcspn(dsn, "=");
-      if (pos != strlen(dsn))
-	{
-	  db_to_use = g_strdup(dsn);
-	}
-      else
-	{
-	  gchar path[128];
-	  g_snprintf(path, sizeof(path), "/gdalib/%s/DSN", dsn);
-	  db_to_use = gnome_config_get_string(path);
-	}
-      if (!db_to_use)
-	{
-	  Gda_Error* e = gda_error_new();
+      Gda_Error* e = gda_error_new();
 	  
-	  e->description = g_strdup_printf(_("Database '%s' not found in system configuration"), dsn);
-	  e->source = g_strdup("[GDA Client Library]");
-	  e->nativeMsg = g_strdup(e->description);
-	  gda_connection_add_single_error(cnc, e);
-	  return -1;
-	}
+      e->description = g_strdup_printf(_("Database '%s' not found in system configuration"), dsn);
+      e->source = g_strdup("[GDA Client Library]");
+      e->nativeMsg = g_strdup(e->description);
+      gda_connection_add_single_error(cnc, e);
+      return -1;
     }
-      			 
-  user_to_use = user;
-  pwd_to_use = pwd;
 
   cnc->database = g_strdup(db_to_use);
-  cnc->user = g_strdup(user_to_use);
-  cnc->passwd = g_strdup(pwd_to_use);
+  cnc->user = g_strdup(user);
+  cnc->passwd = g_strdup(pwd);
 
+  /* start CORBA connection */
+  if (get_corba_connection(cnc) < 0)
+    {
+      Gda_Error* e = gda_error_new();
+
+      e->description = g_strdup(_("Could not open CORBA factory"));
+      e->source = g_strdup("[GDA Client Library]");
+      e->nativeMsg = g_strdup(e->description);
+      gda_connection_add_single_error(cnc, e);
+      return -1;
+    }
+
+  CORBA_exception_init(&ev);
+
+  rc = GDA_Connection_open(cnc->connection, cnc->database, cnc->user, cnc->passwd, &ev);
+  g_print("GDA_Connection_open returns %d\n", rc);
+  if (gda_connection_corba_exception(cnc, &ev) < 0)
+    {
+      CORBA_SystemException* sysexc = CORBA_exception_value(&ev);
+      if (sysexc->completed != CORBA_COMPLETED_NO)
+        {
+          GDA_Connection_close(cnc->connection, &ev);
+          CORBA_Object_release(cnc->connection, &ev);
+        }
+      cnc->connection = CORBA_OBJECT_NIL;
+      return -1;
+    }
+  
+  cnc->is_open = 1;
   gtk_signal_emit(GTK_OBJECT(cnc), gda_connection_signals[CONNECTION_OPEN]);
-  if (!cnc->is_open)
-    return -1 ;
   return 0;
 }
 
@@ -576,6 +530,21 @@ gda_connection_open (Gda_Connection* cnc, gchar* dsn, gchar* user, gchar* pwd)
 void
 gda_connection_close (Gda_Connection* cnc)
 {
+  CORBA_Environment     ev;
+  
+  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+
+  g_return_if_fail(cnc != NULL);
+  g_return_if_fail(gda_connection_is_open(cnc));
+  g_return_if_fail(cnc->connection != NULL);
+
+  CORBA_exception_init(&ev);
+  GDA_Connection_close(cnc->connection, &ev);
+  gda_connection_corba_exception(cnc, &ev);
+  CORBA_Object_release(cnc->connection, &ev);
+  cnc->is_open = 0;
+  cnc->connection = CORBA_OBJECT_NIL;
+
   gtk_signal_emit(GTK_OBJECT(cnc), gda_connection_signals[CONNECTION_CLOSE]);
 }
 
@@ -586,11 +555,11 @@ open_schema (Gda_Connection* cnc, GDA_Connection_QType t,
 {
   CORBA_Environment ev;
   CORBA_Object      corba_rs;
-  Gda_Recordset*     rs;
+  Gda_Recordset*    rs;
   
   CORBA_exception_init(&ev);
 
-  corba_rs = GDA_Connection_openSchema(cnc->connection, t,constraints, &ev);
+  corba_rs = GDA_Connection_openSchema(cnc->connection, t, constraints, &ev);
   if (Exception(&ev) < 0 || corba_rs == CORBA_OBJECT_NIL)
     return 0;
   rs = GDA_RECORDSET(gda_recordset_new());
@@ -620,7 +589,9 @@ open_schema (Gda_Connection* cnc, GDA_Connection_QType t,
  */
 
 Gda_Recordset*
-gda_connection_open_schema_array (Gda_Connection* cnc, GDA_Connection_QType t, Gda_Constraint_Element* elems)
+gda_connection_open_schema_array (Gda_Connection* cnc,
+                                  GDA_Connection_QType t,
+                                  Gda_Constraint_Element* elems)
 {
 
   CORBA_Environment ev;
@@ -628,8 +599,10 @@ gda_connection_open_schema_array (Gda_Connection* cnc, GDA_Connection_QType t, G
   gint                          count = 0;
   gint                          index;
   Gda_Constraint_Element*       current;
+  Gda_Recordset*                rc;
   
   g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
+  g_return_val_if_fail(gda_connection_is_open(cnc), 0);
   g_return_val_if_fail(cnc->connection != 0, 0);
 
   count = 0;
@@ -637,7 +610,7 @@ gda_connection_open_schema_array (Gda_Connection* cnc, GDA_Connection_QType t, G
   while (current)
     {
       if (current->type == GDA_Connection_no_CONSTRAINT)
-	break;
+        break;
       current++;
       count++;
     }
@@ -659,7 +632,9 @@ gda_connection_open_schema_array (Gda_Connection* cnc, GDA_Connection_QType t, G
   fprintf(stderr,"client: gda_connection_open_schema: constraints._maximum = %d\n", constraints->_maximum);
   fprintf(stderr,"                                    constraints._length  = %d\n", constraints->_length);
   
-  return open_schema(cnc, t, constraints);
+  rc = open_schema(cnc, t, constraints);
+  CORBA_free(constraints);
+  return (rc);
 }
 
 /**
@@ -702,7 +677,7 @@ gda_connection_open_schema (Gda_Connection* cnc, GDA_Connection_QType t, ...)
 
       constraint_type = va_arg(ap, int);
       if (constraint_type == GDA_Connection_no_CONSTRAINT)
-	break;
+        break;
       constraint_value = va_arg(ap, char*);
       fprintf(stderr,"gda_connection_open_schema: constraint value = '%s'\n", constraint_value);
       c = g_new0(GDA_Connection_Constraint, 1);
@@ -722,7 +697,8 @@ gda_connection_open_schema (Gda_Connection* cnc, GDA_Connection_QType t, ...)
     {
       memcpy(&constraints->_buffer[index], ptr->data,
 	     sizeof(GDA_Connection_Constraint));
-      fprintf(stderr,"CORBA seq: constraint->value = '%s'\n", constraints->_buffer[index].value);
+      fprintf(stderr,"CORBA seq: constraint->value = '%s'\n",
+              constraints->_buffer[index].value);
       index++;
       count--;
       c = ptr->data;
@@ -730,6 +706,7 @@ gda_connection_open_schema (Gda_Connection* cnc, GDA_Connection_QType t, ...)
       ptr = g_list_next(ptr);
     }
   g_list_free(tmp_constraints);
+
   CORBA_exception_init(&ev);
   rc = open_schema(cnc, t, constraints);
   CORBA_free(constraints);
@@ -769,17 +746,17 @@ gda_connection_get_errors (Gda_Connection* cnc)
       remote_errors = GDA_Connection__get_errors(cnc->connection, &ev);
       Exception(&ev);
       for (idx = 0; idx < remote_errors->_length; idx++)
-	{
-	  Gda_Error* e;
-	  e = gda_error_new();
-	  e->description = g_strdup(remote_errors->_buffer[idx].description);
-	  e->number      = remote_errors->_buffer[idx].number;
-	  e->source      = g_strdup(remote_errors->_buffer[idx].source);
-	  e->helpurl = 0;
-	  e->sqlstate    = g_strdup(remote_errors->_buffer[idx].sqlstate);
-	  e->nativeMsg   = g_strdup(remote_errors->_buffer[idx].nativeMsg);
-	  gda_connection_add_single_error(cnc, e);
-	}
+        {
+          Gda_Error* e;
+          e = gda_error_new();
+          e->description = g_strdup(remote_errors->_buffer[idx].description);
+          e->number      = remote_errors->_buffer[idx].number;
+          e->source      = g_strdup(remote_errors->_buffer[idx].source);
+          e->helpurl = 0;
+          e->sqlstate    = g_strdup(remote_errors->_buffer[idx].sqlstate);
+          e->nativeMsg   = g_strdup(remote_errors->_buffer[idx].nativeMsg);
+          gda_connection_add_single_error(cnc, e);
+        }
       CORBA_free(remote_errors);
     }
   
@@ -844,7 +821,6 @@ gda_connection_commit_transaction (Gda_Connection* cnc)
  *
  * Returns: 0 if everything is okay, -1 on error
  */
-
 gint
 gda_connection_rollback_transaction (Gda_Connection* cnc)
 {
@@ -861,10 +837,9 @@ gda_connection_rollback_transaction (Gda_Connection* cnc)
   return 0;
 }
 
-
 /**
  * gda_connection_execute:
- * @cnc: the conneciton
+ * @cnc: the connection
  * @txt: the statement to execute
  * @reccount: a pointer to a gulong. The number of affected records is 
  * stored in the space where the pointer points to
@@ -950,8 +925,7 @@ gda_connection_stop_logging (Gda_Connection* cnc)
     }
   return 0;
 }
-  
-    
+   
 gchar*
 gda_connection_create_recordset (Gda_Connection* cnc, Gda_Recordset* rs)
 {
@@ -964,9 +938,8 @@ gda_connection_create_recordset (Gda_Connection* cnc, Gda_Recordset* rs)
   g_return_val_if_fail(IS_GDA_RECORDSET(rs), 0);
   
   CORBA_exception_init(&ev);
-  retval = GDA_Connection_createTable(cnc->connection,
-				      rs->name,
-				      rs->field_attributes, &ev);
+  retval = GDA_Connection_createTable(cnc->connection, rs->name,
+                                      rs->field_attributes, &ev);
   if (Exception(&ev))
     return 0;
   return retval;
@@ -981,7 +954,8 @@ gda_connection_add_single_error (Gda_Connection* cnc, Gda_Error* error)
   g_return_if_fail(error != 0);
 
   error_list = g_list_append(error_list, error);
-  gtk_signal_emit(GTK_OBJECT(cnc), gda_connection_signals[CONNECTION_ERROR], error_list);
+  gtk_signal_emit(GTK_OBJECT(cnc),
+                  gda_connection_signals[CONNECTION_ERROR], error_list);
 }
 
 void
@@ -1103,4 +1077,4 @@ gda_connection_get_version (Gda_Connection *cnc)
   g_return_val_if_fail(IS_GDA_CONNECTION(cnc), 0);
   g_error("%s: not implemented", __PRETTY_FUNCTION__);
   return (0);
-}
\ No newline at end of file
+}
