@@ -40,13 +40,13 @@ gda_error_get_type (void)
     {
       GtkTypeInfo gda_error_info =
       {
-	"GdaError",
-	sizeof (Gda_Error),
-	sizeof (Gda_ErrorClass),
-	(GtkClassInitFunc) gda_error_class_init,
-	(GtkObjectInitFunc) gda_error_init,
-	(GtkArgSetFunc) NULL,
-	(GtkArgSetFunc) NULL,
+        "GdaError",
+        sizeof (Gda_Error),
+        sizeof (Gda_ErrorClass),
+        (GtkClassInitFunc) gda_error_class_init,
+        (GtkObjectInitFunc) gda_error_init,
+        (GtkArgSetFunc) NULL,
+        (GtkArgSetFunc) NULL,
       };
       gda_error_type = gtk_type_unique(gtk_object_get_type(), &gda_error_info);
     }
@@ -82,7 +82,7 @@ gda_error_init (Gda_Error* error)
  *
  * Returns: the error object.
  */
-Gda_Error*
+Gda_Error *
 gda_error_new (void)
 {
   Gda_Error* error;
@@ -105,56 +105,57 @@ gda_errors_from_exception (CORBA_Environment* ev)
       return 0;
     case CORBA_SYSTEM_EXCEPTION:
       {
-	CORBA_SystemException* sysexc;
+        CORBA_SystemException* sysexc;
 	
-	sysexc = CORBA_exception_value(ev);
-	error = gda_error_new();
-	error->source = g_strdup("[CORBA System Exception]");
-	switch(sysexc->minor)
-	  {
-	  case ex_CORBA_COMM_FAILURE:
-	    error->description = g_strdup_printf(_("%s: The server didn't respond."), CORBA_exception_id(ev));
-	    break;
-	  default:
-	    error->description = g_strdup(_("%s: An Error occured in the CORBA system."));
-	    break;
-	  }
-	all_errors = g_list_append(all_errors, error);
+        sysexc = CORBA_exception_value(ev);
+        error = gda_error_new();
+        error->source = g_strdup("[CORBA System Exception]");
+        switch(sysexc->minor)
+          {
+          case ex_CORBA_COMM_FAILURE:
+            error->description = g_strdup_printf(_("%s: The server didn't respond."),
+                                                 CORBA_exception_id(ev));
+            break;
+          default:
+            error->description = g_strdup(_("%s: An Error occured in the CORBA system."));
+            break;
+          }
+        all_errors = g_list_append(all_errors, error);
       }
     case CORBA_USER_EXCEPTION:
       {
-	if (strcmp(CORBA_exception_id(ev), ex_GDA_NotSupported) == 0)
-	  {
-	    GDA_NotSupported* not_supported_error;
+        if (strcmp(CORBA_exception_id(ev), ex_GDA_NotSupported) == 0)
+          {
+            GDA_NotSupported* not_supported_error;
 	    
-	    not_supported_error = (GDA_NotSupported*)ev->_params;
-	    error = gda_error_new();
-	    error->source = g_strdup("[CORBA User Exception]");
-	    error->description = g_strdup(not_supported_error->errormsg);
-	    all_errors = g_list_append(all_errors, error);
-	  }
-	else if (strcmp(CORBA_exception_id(ev), ex_GDA_DriverError) == 0)
-	  {
-	    GDA_ErrorSeq  error_sequence = ((GDA_DriverError*)ev->_params)->errors;
-	    gint          idx;
+            not_supported_error = (GDA_NotSupported*)ev->_params;
+            error = gda_error_new();
+            error->source = g_strdup("[CORBA User Exception]");
+            error->description = g_strdup(not_supported_error->errormsg);
+            all_errors = g_list_append(all_errors, error);
+          }
+        else if (strcmp(CORBA_exception_id(ev), ex_GDA_DriverError) == 0)
+          {
+            GDA_ErrorSeq  error_sequence = ((GDA_DriverError*)ev->_params)->errors;
+            gint          idx;
 	    
-	    for (idx = 0; idx < error_sequence._length; idx++)
-	      {
-		GDA_Error* gda_error = &error_sequence._buffer[idx];
+            for (idx = 0; idx < error_sequence._length; idx++)
+              {
+                GDA_Error* gda_error = &error_sequence._buffer[idx];
 		
-		error = gda_error_new();
-		if (gda_error->source)
-		  error->source = g_strdup(gda_error->source);
-		error->number = gda_error->number;
-		if (gda_error->sqlstate)
-		  error->sqlstate = g_strdup(gda_error->sqlstate);
-		if (gda_error->nativeMsg)
-		  error->nativeMsg = g_strdup(gda_error->nativeMsg);
-		if (gda_error->description)
-		  error->description = g_strdup(gda_error->description);
-		all_errors = g_list_append(all_errors, error);
-	      }
-	  }
+                error = gda_error_new();
+                if (gda_error->source)
+                  error->source = g_strdup(gda_error->source);
+                error->number = gda_error->number;
+                if (gda_error->sqlstate)
+                  error->sqlstate = g_strdup(gda_error->sqlstate);
+                if (gda_error->nativeMsg)
+                  error->nativeMsg = g_strdup(gda_error->nativeMsg);
+                if (gda_error->description)
+                  error->description = g_strdup(gda_error->description);
+                all_errors = g_list_append(all_errors, error);
+              }
+          }
       }
       break;
     default:
@@ -206,7 +207,7 @@ gda_error_list_free (GList* errors)
 
   g_return_if_fail(errors != 0);
   
-  while(ptr)
+  while (ptr)
     {
       Gda_Error* error = ptr->data;
       gda_error_free(error);
