@@ -1,6 +1,7 @@
 /* GNOME DB libary
  * Copyright (C) 1998,1999 Michael Lausch
  * Copyright (C) 1999 Rodrigo Moya
+ * Copyright (C) 2000 Vivien Malerba
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -44,13 +45,13 @@ gda_field_get_type (void)
     {
       GtkTypeInfo gda_field_info =
       {
-	"GdaField",
-	sizeof (Gda_Field),
-	sizeof (Gda_FieldClass),
-	(GtkClassInitFunc) gda_field_class_init,
-	(GtkObjectInitFunc) gda_field_init,
-	(GtkArgSetFunc) NULL,
-	(GtkArgSetFunc) NULL,
+        "GdaField",
+        sizeof (Gda_Field),
+        sizeof (Gda_FieldClass),
+        (GtkClassInitFunc) gda_field_class_init,
+        (GtkObjectInitFunc) gda_field_init,
+        (GtkArgSetFunc) NULL,
+        (GtkArgSetFunc) NULL,
       };
       gda_field_type = gtk_type_unique(gtk_object_get_type(), &gda_field_info);
     }
@@ -79,7 +80,6 @@ gda_field_init (Gda_Field* field)
  *
  * Returns: a pointer to the buffer.
  */
-
 gchar* 
 gda_fieldtype_2_string(gchar* bfr, gint length, GDA_ValueType type)
 {
@@ -120,6 +120,7 @@ gda_fieldtype_2_string(gchar* bfr, gint length, GDA_ValueType type)
       ENUM_TO_STR(GDA_TypeFixchar);
       ENUM_TO_STR(GDA_TypeFixbin);
       ENUM_TO_STR(GDA_TypeFixwchar);
+      ENUM_TO_STR(GDA_TypeLastValue);
     }
   return bfr;
 }
@@ -174,7 +175,9 @@ gchar*
 gda_stringify_value(gchar* bfr, gint maxlen, Gda_Field* f)
 {
   gchar*     retval = 0;
-  
+  gchar      tmp[40];
+  struct tm  stm;
+
   g_return_val_if_fail(IS_GDA_FIELD(f), 0);
   
   if (bfr)
@@ -187,131 +190,175 @@ gda_stringify_value(gchar* bfr, gint maxlen, Gda_Field* f)
     {
     case GDA_TypeNull:
       if (bfr)
-	retval = strncpy(bfr, _("<Unknown GDA Type(NULL)>"), maxlen);
+        retval = strncpy(bfr, _("<Unknown GDA Type(NULL)>"), maxlen);
       else
-	retval = g_strdup(_("<Unknown GDA Type(NULL)>"));
+        retval = g_strdup(_("<Unknown GDA Type(NULL)>"));
       break;
     case GDA_TypeBigint:
       if (!bfr)
-	{
-	  retval = g_new0(gchar,20);
-	  maxlen = 20;
-	}
+        {
+          retval = g_new0(gchar,20);
+          maxlen = 20;
+        }
       g_snprintf(retval, maxlen, "%Ld", gda_field_bigint(f));
       break;
     case GDA_TypeBoolean:
       {
-	gchar* retstr;
-	if (gda_field_boolean(f))
-	  retstr = _("TRUE");
-	else
-	  retstr = _("FALSE");
-	if (bfr)
-	  retval = strncpy(bfr, retstr, maxlen);
-	else
-	  retval = g_strdup(retstr);
+        gchar* retstr;
+        if (gda_field_boolean(f)) retstr = _("TRUE");
+        else retstr = _("FALSE");
+
+        if (bfr) retval = strncpy(bfr, retstr, maxlen);
+        else retval = g_strdup(retstr);
       }
       break;
     case GDA_TypeVarbin:
       {
-	gint len = maxlen - 1;
-	gint copylen;
+        gint len = maxlen - 1;
+        gint copylen;
 	
-	if (!bfr)
-	  {
-	    retval = g_new0(char, f->real_value->_u.v._u.lvb._length + 1);
-	    len = f->real_value->_u.v._u.lvb._length;
-	  }
-	copylen = min(len, f->real_value->_u.v._u.lvb._length);
-	fprintf(stderr,"GDA_TypeVarbin: Copying %d bytes\n", copylen);
-	memcpy(retval, f->real_value->_u.v._u.lvb._buffer, copylen);
-	retval[len] = '\0';
+        if (!bfr)
+          {
+            retval = g_new0(char, f->real_value->_u.v._u.lvb._length + 1);
+            len = f->real_value->_u.v._u.lvb._length;
+          }
+        copylen = min(len, f->real_value->_u.v._u.lvb._length);
+        fprintf(stderr,"GDA_TypeVarbin: Copying %d bytes\n", copylen);
+        memcpy(retval, f->real_value->_u.v._u.lvb._buffer, copylen);
+        retval[len] = '\0';
       }
       break;
     case GDA_TypeLongvarchar:
     case GDA_TypeVarchar:
       if (!bfr)
-	{
-	  retval = g_strdup(f->real_value->_u.v._u.lvc);
-	}
+        {
+          retval = g_strdup(f->real_value->_u.v._u.lvc);
+        }
       else
-	{
-	  strncpy(retval, f->real_value->_u.v._u.lvc,
-		  min(maxlen, strlen(f->real_value->_u.v._u.lvc)));
-	  retval[min(maxlen, strlen(f->real_value->_u.v._u.lvc))] = '\0';
-	}
+        {
+          strncpy(retval, f->real_value->_u.v._u.lvc,
+                  min(maxlen, strlen(f->real_value->_u.v._u.lvc)));
+          retval[min(maxlen, strlen(f->real_value->_u.v._u.lvc))] = '\0';
+        }
       break;
     case GDA_TypeInteger:
+      if (!bfr)
+        {
+          retval = g_new0(gchar, 20);
+          maxlen = 20;
+        }
+      g_snprintf(retval, maxlen, "%d", f->real_value->_u.v._u.i);
+      break;
     case GDA_TypeSmallint:
       if (!bfr)
-	{
-	  retval = g_new0(gchar, 20);
-	  maxlen = 20;
-	}
+        {
+          retval = g_new0(gchar, 20);
+          maxlen = 20;
+        }
       g_snprintf(retval, maxlen, "%d", f->real_value->_u.v._u.si);
       break;
     case GDA_TypeChar:
       if (!bfr)
-	retval = g_strdup(f->real_value->_u.v._u.lvc);
+        retval = g_strdup(f->real_value->_u.v._u.lvc);
       else
-	{
-	  strncpy(retval, f->real_value->_u.v._u.lvc,
-		  min(maxlen, strlen(f->real_value->_u.v._u.lvc)));
-	  retval[min(maxlen, strlen(f->real_value->_u.v._u.lvc))] = '\0';
-	}
+        {
+          strncpy(retval, f->real_value->_u.v._u.lvc,
+                  min(maxlen, strlen(f->real_value->_u.v._u.lvc)));
+          retval[min(maxlen, strlen(f->real_value->_u.v._u.lvc))] = '\0';
+        }
       break;
     case GDA_TypeDecimal:
     case GDA_TypeNumeric:
       if (!bfr)
-	retval = g_strdup(f->real_value->_u.v._u.lvc);
+        retval = g_strdup(f->real_value->_u.v._u.lvc);
       else
-	{
-	  strncpy(retval, f->real_value->_u.v._u.lvc,
-		  min(maxlen, strlen(f->real_value->_u.v._u.lvc)));
-	  retval[min(maxlen, strlen(f->real_value->_u.v._u.lvc))] = '\0';
-	}
+        {
+          strncpy(retval, f->real_value->_u.v._u.lvc,
+                  min(maxlen, strlen(f->real_value->_u.v._u.lvc)));
+          retval[min(maxlen, strlen(f->real_value->_u.v._u.lvc))] = '\0';
+        }
       break;
     case GDA_TypeSingle:
       if (!bfr)
-	{
-	  retval = g_new0(gchar, 20);
-	  maxlen = 20;
-	}
+        {
+          retval = g_new0(gchar, 20);
+          maxlen = 20;
+        }
       g_snprintf(retval, maxlen, "%g", f->real_value->_u.v._u.f);
       break;
     case GDA_TypeDouble:
       if (!bfr)
-	{
-	  retval = g_new0(gchar, 20);
-	  maxlen = 20;
-	}
+        {
+          retval = g_new0(gchar, 20);
+          maxlen = 20;
+        }
       g_snprintf(retval, maxlen, "%g", f->real_value->_u.v._u.dp);
       break;
+    case GDA_TypeDbTime:
+      stm.tm_sec = f->real_value->_u.v._u.dbt.second;
+      stm.tm_min = f->real_value->_u.v._u.dbt.minute;
+      stm.tm_hour = f->real_value->_u.v._u.dbt.hour;
+      strftime(tmp, 30, "%X", &stm);
+      if (!bfr)
+        retval = g_strdup(tmp);
+      else
+        {
+          strncpy(retval, tmp, min(maxlen, strlen(tmp)));
+          retval[min(maxlen, strlen(tmp))] = '\0';
+        }
+      break;
+    case GDA_TypeDbDate:
+      stm.tm_year = f->real_value->_u.v._u.dbd.year-1900;
+      stm.tm_mon = f->real_value->_u.v._u.dbd.month-1;
+      stm.tm_mday = f->real_value->_u.v._u.dbd.day;
+      strftime(tmp, 30, "%x", &stm);
+      if (!bfr)
+        retval = g_strdup(tmp);
+      else
+        {
+          strncpy(retval, tmp, min(maxlen, strlen(tmp)));
+          retval[min(maxlen, strlen(tmp))] = '\0';
+        }
+      break;
+    case GDA_TypeDbTimestamp:
+      stm.tm_sec = f->real_value->_u.v._u.dbtstamp.second;
+      stm.tm_min = f->real_value->_u.v._u.dbtstamp.minute;
+      stm.tm_hour = f->real_value->_u.v._u.dbtstamp.hour;
+      stm.tm_year = f->real_value->_u.v._u.dbtstamp.year-1900;
+      stm.tm_mon = f->real_value->_u.v._u.dbtstamp.month-1;
+      stm.tm_mday = f->real_value->_u.v._u.dbtstamp.day;
+      strftime(tmp, 40, "%x %X", &stm); /* the %c does not seem to work! */
+      if (!bfr)
+        retval = g_strdup(tmp);
+      else
+        {
+          strncpy(retval, tmp, min(maxlen, strlen(tmp)));
+          retval[min(maxlen, strlen(tmp))] = '\0';
+        }
+      break;
     case GDA_TypeBinary:
     case GDA_TypeBstr:
     case GDA_TypeCurrency:
     case GDA_TypeDate:
-    case GDA_TypeDbDate:
-    case GDA_TypeDbTime:
-    case GDA_TypeDbTimestamp:
     case GDA_TypeError:
     case GDA_TypeLongvarbin:
     case GDA_TypeLongvarwchar:
-      
     case GDA_TypeTinyint:
     case GDA_TypeUBigint:
     case GDA_TypeUSmallint:
     case GDA_TypeVarwchar:
     case GDA_TypeFixchar:
     case GDA_TypeFixbin:
+    case GDA_TypeLastValue:
     case GDA_TypeFixwchar:
       if (!bfr)
-	{
-	  retval = g_new0(gchar, 128);
-	  maxlen = 128;
-	}
-      printf("stringify for valuetype [%d]'%s' NYI\n",
-	     f->attributes->gdaType, gda_fieldtype_2_string(0, 20, gda_field_type(f)));
+        {
+          retval = g_new0(gchar, 128);
+          maxlen = 128;
+        }
+      g_print("stringify for valuetype [%d]'%s' NYI\n",
+              f->attributes->gdaType,
+              gda_fieldtype_2_string(0, 20, gda_field_type(f)));
       retval[0] = '\0';
       break;
     }
@@ -324,8 +371,7 @@ gda_stringify_value(gchar* bfr, gint maxlen, Gda_Field* f)
  *
  * Returns: the pointer to the new field object.
  */
-
-Gda_Field*
+Gda_Field *
 gda_field_new (void)
 {
   Gda_Field* rc = GDA_FIELD(gtk_type_new(gda_field_get_type()));
@@ -340,7 +386,6 @@ gda_field_new (void)
  * Free the memory allocated for this field object.
  *
  */
-
 void
 gda_field_free (Gda_Field* f)
 {
@@ -348,7 +393,6 @@ gda_field_free (Gda_Field* f)
   g_free(f);
 }
 
-
 /**
  * gda_field_actual_size:
  * @f: a pointer to the field.
@@ -358,7 +402,6 @@ gda_field_free (Gda_Field* f)
  * Returns: the number of bytes the value fo the field needs or 0 if
  * the field has a NULL value.
  */
-
 gint
 gda_field_actual_size (Gda_Field* f)
 {
@@ -413,8 +456,7 @@ gda_field_actual_size (Gda_Field* f)
     case GDA_TypeUSmallint:
       return sizeof(CORBA_unsigned_short);
     default:
-      g_warning("gda_field_actual_size: unknown GDA Type %d\n", gda_field_type(f));
-      return -1;
+      break;
     }
   g_warning("gda_field_actual_size: unknown GDA Type %d\n", gda_field_type(f));
   return -1;
