@@ -17,13 +17,7 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <gnome.h>
-#include <gda.h>
-
-#include "gda-recordset.h"
-#include "gda-command.h"
-#include "gda-commandP.h"
-#include "gda-error.h"
+#include <gda-connection.h>
 
 enum
 {
@@ -49,21 +43,12 @@ static gulong fetch_and_dont_store     (Gda_Recordset* rs, gint count, gpointer
 /* per default fetch 64 rows at a time */
 #define GDA_DEFAULT_CACHESIZE 64
 
-#define __DEBUG_NEW_CODE 1
-
 static void
 gda_recset_value_changed (GtkAdjustment* adj)
 {
   Gda_Recordset* rs = GDA_RECORDSET(adj);
   gint           offset;
 
-#if __DEBUG_NEW_CODE
-  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
-  g_print("\tadj->value  = %g\n", adj->value);
-  g_print("\trs->current = %ld\n", rs->current_index);
-
-#endif
-
   if (adj->value < 1.0)
     adj->value = 1.0;
   
@@ -77,14 +62,10 @@ gda_recset_value_changed (GtkAdjustment* adj)
 
   if (offset == 0)
     {
-#if __DEBUG_NEW_CODE
       g_print("\toffset == 0, No action taken\n");
-#endif      
       return;
     }
-#if __DEBUG_NEW_CODE
-    g_print("\toffset = %d\n", (gint)offset);
-#endif
+  g_print("\toffset = %d\n", (gint)offset);
     
   if (rs->cursor_location == GDA_USE_CLIENT_CURSOR)
     fetch_and_store(rs, offset, 0);
@@ -93,9 +74,7 @@ gda_recset_value_changed (GtkAdjustment* adj)
 
   adj->value = rs->current_index;
 
-#if __DEBUG_NEW_CODE
   g_print("\tnew adjustment value = %g\n", adj->value);
-#endif;  
   if (gda_recordset_eof(rs))
     {
       adj->upper = adj->value;
@@ -106,9 +85,9 @@ static void
 gda_recordset_real_error (Gda_Recordset *rs, GList *error_list)
 {
   Gda_Connection *cnc;
-#if __DEBUG_NEW_CODE
-  g_print("%s: %d: %s called\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
-#endif
+
+  g_return_if_fail(IS_GDA_CONNECTION(cnc));
+
   if ((cnc = gda_recordset_get_connection(rs)))
     {
       gda_connection_add_errorlist(cnc, error_list);
@@ -124,15 +103,16 @@ gda_recordset_get_type (void)
     {
       GtkTypeInfo gda_recordset_info =
       {
-	"GdaRecordset",
-	sizeof(Gda_Recordset),
-	sizeof(Gda_RecordsetClass),
-	(GtkClassInitFunc) gda_recordset_class_init,
-	(GtkObjectInitFunc) gda_recordset_init,
-	(GtkArgSetFunc) NULL,
-	(GtkArgSetFunc) NULL
+        "GdaRecordset",
+        sizeof(Gda_Recordset),
+        sizeof(Gda_RecordsetClass),
+        (GtkClassInitFunc) gda_recordset_class_init,
+        (GtkObjectInitFunc) gda_recordset_init,
+        (GtkArgSetFunc) NULL,
+        (GtkArgSetFunc) NULL
       };
-      gda_recordset_type = gtk_type_unique(gtk_adjustment_get_type(), &gda_recordset_info);
+      gda_recordset_type = gtk_type_unique(gtk_adjustment_get_type(),
+                                           &gda_recordset_info);
     }
   return (gda_recordset_type);
 }
@@ -191,6 +171,7 @@ gda_recordset_init (Gda_Recordset *rs)
   rs->field_attributes = 0;
   rs->max_index        = 0;
   rs->maxrows          = 0;
+  rs->affected_rows    = 0;
   rs->cachesize        = 0;
   rs->open             = 0;
   rs->eof              = 1;
@@ -239,7 +220,6 @@ row_by_idx (Gda_Recordset* rs, gint idx)
  *
  * Returns: the allocated recordset object
  */
-
 GtkObject*
 gda_recordset_new (void)
 {
@@ -372,7 +352,7 @@ fetch_and_store(Gda_Recordset* rs, gint count, gpointer bookmark)
 
   if (count < 0)		
     {
-      /* Going backward  and underflow condition */      
+      /* going backward  and underflow condition */      
       rs->eof = 0;
       rs->bof = 1;
       return rs->current_index;
@@ -380,32 +360,33 @@ fetch_and_store(Gda_Recordset* rs, gint count, gpointer bookmark)
 
   rs->eof = 0;			/* be optimistic */
   rs->bof = 0;
+
   /* going forward. We have to fetch it from the provider */
   CORBA_exception_init(&ev);
   for (current_count = 0; current_count < count; current_count++)
     {
       chunk = GDA_Recordset_fetch(rs->corba_rs, 1, &ev);
       if (gda_connection_corba_exception(rs->cnc, &ev))
-	{
-	  /* An error */
-	  return rs->current_index;
-	}
+        {
+          /* An error */
+          return rs->current_index;
+        }
       if (chunk->_length == 0)
-	{
-	  /* EOF. No more records there */
-	  CORBA_free(chunk);
-	  rs->current_index = rs->current_index + current_count;
-	  rs->max_index = rs->current_index;
-	  rs->eof = 1;
-	  if (!rs->current_row)
-	    rs->bof = 1;
-	  return rs->current_index;
-	}
+        {
+          /* EOF. No more records there */
+          CORBA_free(chunk);
+          rs->current_index = rs->current_index + current_count;
+          rs->max_index = rs->current_index;
+          rs->eof = 1;
+          if (!rs->current_row)
+            rs->bof = 1;
+          return rs->current_index;
+        }
       rs->chunks = g_list_append(rs->chunks, chunk);
       if (gda_recordset_eof(rs))
-	{
-	  current_idx--;
-	}
+        {
+          current_idx--;
+        }
     }
   rs->max_index = current_idx;
   rs->current_index = current_idx;
@@ -518,14 +499,13 @@ gda_recordset_move (Gda_Recordset* rs, gint count, gpointer bookmark)
 gulong
 gda_recordset_move_first (Gda_Recordset* rs)
 {
-
   g_return_val_if_fail(IS_GDA_RECORDSET(rs), GDA_RECORDSET_INVALID_POSITION);
   
   if (rs->cursor_location == GDA_USE_CLIENT_CURSOR)
     {
       GDA_Recordset_Chunk* chunk;
       if (!rs->chunks)
-	return gda_recordset_move(rs, 1, 0);
+        return gda_recordset_move(rs, 1, 0);
       rs->current_index = 1;
       chunk = rs->chunks->data;
       rs->current_row = &chunk->_buffer[0];
@@ -561,15 +541,15 @@ gda_recordset_move_last (Gda_Recordset* rs)
     {
       pos = rs->current_index;
       while (1)
-	{
-	  rc = gda_recordset_move(rs, 1, 0);
+        {
+          rc = gda_recordset_move(rs, 1, 0);
 	  
-	  if (rc == GDA_RECORDSET_INVALID_POSITION)
-	    return GDA_RECORDSET_INVALID_POSITION;
-	  if (gda_recordset_eof(rs))
-	    return pos;
-	  pos++;
-	}
+          if (rc == GDA_RECORDSET_INVALID_POSITION)
+            return GDA_RECORDSET_INVALID_POSITION;
+          if (gda_recordset_eof(rs))
+           return pos;
+          pos++;
+        }
     }
   g_error("gda_recordset_move_first for server based cursor NIY");
   return GDA_RECORDSET_INVALID_POSITION;
@@ -646,7 +626,7 @@ gda_recordset_field_idx (Gda_Recordset* rs, gint idx)
  * Returns: a pointer to a field structor or NULL if 
  * @idx is out of range
  */
-Gda_Field*
+Gda_Field *
 gda_recordset_field_name (Gda_Recordset* rs, gchar* name)
 {
   gint        rowsize;
@@ -693,9 +673,22 @@ gda_recordset_rowsize (Gda_Recordset* rs)
   return rs->field_attributes->_length;
 }
 
+/**
+ * gda_recordset_affected_rows
+ * @rs: the recordset
+ *
+ * Return the number of affected rows in the recordset
+ */
+gulong
+gda_recordset_affected_rows (Gda_Recordset *rs)
+{
+  g_return_val_if_fail(IS_GDA_RECORDSET(rs), 0);
+  return rs->affected_rows;
+}
+
 /**
  * gda_recordset_set_connection:
- * @rs: the rcordset
+ * @rs: the recordset
  * @cnc: the conneciton
  *
  * Associates a recordset with a connection. This function must not be 
@@ -706,7 +699,6 @@ gda_recordset_rowsize (Gda_Recordset* rs)
  *
  * Returns: 0 if everyhting is okay, -1 on error
  */
-
 gint
 gda_recordset_set_connection (Gda_Recordset* rs, Gda_Connection* cnc)
 {
@@ -758,8 +750,8 @@ gint
 gda_recordset_open (Gda_Recordset* rs,
                     Gda_Command* cmd,
                     GDA_CursorType cursor_type,
-		    GDA_LockType lock_type,
-		    gulong options)
+                    GDA_LockType lock_type,
+                    gulong options)
 {
   CORBA_Environment  ev;
   GDA_CmdParameterSeq*      corba_parameters;
@@ -793,6 +785,7 @@ gda_recordset_open (Gda_Recordset* rs,
       return -1;
     }
   rs->open = 1;
+  rs->affected_rows = affected;
   return 0;
 }
 
@@ -818,8 +811,8 @@ gint
 gda_recordset_open_txt (Gda_Recordset* rs,
                         gchar* txt,
                         GDA_CursorType cursor_type,
-		        GDA_LockType lock_type,
-		        gulong options)
+                        GDA_LockType lock_type,
+                        gulong options)
 {
   CORBA_Environment  ev;
   Gda_Command*       cmd;
@@ -854,10 +847,11 @@ gda_recordset_add_field (Gda_Recordset* rs, Gda_Field* field)
       gint                 idx;
 
       for (idx = 0; idx < rs->field_attributes->_length; idx++)
-	{
-	  if (strcasecmp(rs->field_attributes->_buffer[idx].name, field->attributes->name) == 0)
-	    return -1;
-	}
+        {
+          if (strcasecmp(rs->field_attributes->_buffer[idx].name,
+                         field->attributes->name) == 0)
+            return -1;
+        }
       rs->field_attributes->_length++;
       old = rs->field_attributes->_buffer;
       rs->field_attributes->_buffer = CORBA_sequence_GDA_FieldAttributes_allocbuf(rs->field_attributes->_length);
