@@ -28,8 +28,8 @@
 /*
  * Database interface
  */
-extern void gb_database_connect    (GtkWidget *w, gpointer data);
-extern void gb_database_disconnect (GtkWidget *w, gpointer data);
+extern void builder_database_connect    (GtkWidget *w, gpointer data);
+extern void builder_database_disconnect (GtkWidget *w, gpointer data);
 
 /*
  * Global variables (defined in gda-builder-main.c)
