@@ -19,8 +19,12 @@
 #ifndef __GDA_MYSQL_COMMAND_H__
 #define __GDA_MYSQL_COMMAND_H__
 
+#include "gda.h"
 #include "gda-mysql-types.h"
 
-
+Gda_MYSQL_Command*    gda_mysql_cmd_new            (void);
+void                  gda_mysql_cmd_free           (Gda_MYSQL_Command* cmd);
+Gda_MYSQL_Connection* gda_mysql_cmd_set_connection (Gda_MYSQL_Command* cmd, Gda_MYSQL_Connection* cnc);
+Gda_MYSQL_Recordset*  gda_mysql_cmd_execute        (Gda_MYSQL_Command* cmd, const GDA_CmdParameterSeq* params, gulong options);
 
 #endif
