@@ -24,43 +24,40 @@
 #include <mysql/mysql.h>
 #include <glib.h>
 
-
-
 #include "gda-mysql-connection.h"
 #include "gda-mysql-recset.h"
 
 typedef Gda_MYSQL_Recordset* (*schema_ops_fn) (Gda_MYSQL_Recordset*,
-					       Gda_MYSQL_Connection*,
-					       GDA_Connection_Constraint* constraints,
-					       gint length);
+                                               Gda_MYSQL_Connection*,
+                                               GDA_Connection_Constraint* constraints,
+                                               gint length);
 
-schema_ops_fn schema_ops[GDNC_SCHEMA_LAST] =
-{
-  0
-};
+schema_ops_fn schema_ops[GDA_Connection_GDCN_SCHEMA_LAST] = { 0 };
 
 static Gda_MYSQL_Recordset* schema_tables(Gda_MYSQL_Recordset* recset,
-					  Gda_MYSQL_Connection*,
-					  GDA_Connection_Constraint* constraints,
-					  gint length);
+                                          Gda_MYSQL_Connection*,
+                                          GDA_Connection_Constraint* constraints,
+                                          gint length);
 
 static Gda_MYSQL_Recordset* schema_views(Gda_MYSQL_Recordset* recset,
-					 Gda_MYSQL_Connection*,
-					 GDA_Connection_Constraint* constraints,
-					 gint length);
+                                         Gda_MYSQL_Connection*,
+                                         GDA_Connection_Constraint* constraints,
+                                         gint length);
 static Gda_MYSQL_Recordset* schema_cols(Gda_MYSQL_Recordset* recset,
-					Gda_MYSQL_Connection*,
-					GDA_Connection_Constraint* constraints,
-					gint length);
+                                        Gda_MYSQL_Connection*,
+                                        GDA_Connection_Constraint* constraints,
+                                        gint length);
 
 void           initialize_schema_ops(void);
 
 void
 initialize_schema_ops(void)
 {
-  schema_ops[GDCN_SCHEMA_TABLES] = schema_tables;
-  schema_ops[GDCN_SCHEMA_VIEWS]  = schema_views;
-  schema_ops[GDCN_SCHEMA_COLS]   = schema_cols;
+  memset((gpointer) schema_ops, 0, sizeof(schema_ops));
+
+  schema_ops[GDA_Connection_GDCN_SCHEMA_TABLES] = schema_tables;
+  schema_ops[GDA_Connection_GDCN_SCHEMA_VIEWS]  = schema_views;
+  schema_ops[GDA_Connection_GDCN_SCHEMA_COLS]   = schema_cols;
 }
 
 
@@ -113,21 +110,21 @@ static Gda_MYSQL_Recordset*
 schema_views(Gda_MYSQL_Recordset* recset, Gda_MYSQL_Connection* cnc,
 	      GDA_Connection_Constraint* constraints, gint length)
 {
-  return 0;
+  return recset;
 }
 
 static Gda_MYSQL_Recordset*
 schema_cols(Gda_MYSQL_Recordset* recset, Gda_MYSQL_Connection* cnc,
 	      GDA_Connection_Constraint* constraints, gint length)
 {
-  return 0;
+  return recset;
 }
 
 
 Gda_MYSQL_Connection*
 gda_mysql_connection_new()
 {
-  static int initialized = 0;
+  static gint initialized = 0;
   
   Gda_MYSQL_Connection* c = g_new0(Gda_MYSQL_Connection, 1);
 
@@ -242,9 +239,7 @@ gda_mysql_connection_open(Gda_MYSQL_Connection* cnc,
   
   if (!rc)
     {
-      fprintf(stderr,"mysql errno  = %d\n",mysql_errno(cnc->mysql));
-      fprintf(stderr,"mysql errmsg = '%s'\n",
-	      mysql_error(cnc->mysql));
+      gda_mysql_error_make(cnc, "gda_mysql_connection_open");
       return -1;
     }
 
@@ -268,11 +263,11 @@ gda_mysql_connection_open(Gda_MYSQL_Connection* cnc,
 }
 
 Gda_MYSQL_Recordset*
-gda_mysql_connection_open_schema(Gda_MYSQL_Connection*      cnc,
-				 Gda_MYSQL_Error*           e,
-				 GDA_Connection_QType       t,
-				 GDA_Connection_Constraint* constraints,
-				 gint  			    length)
+gda_mysql_connection_open_schema (Gda_MYSQL_Connection* cnc,
+				  Gda_MYSQL_Error* e,
+				  GDA_Connection_QType t,
+				  GDA_Connection_Constraint* constraints,
+				  gint length)
 {
   Gda_MYSQL_Recordset* rc = gda_mysql_recset_new();
   schema_ops_fn        fn = schema_ops[t];
@@ -285,16 +280,31 @@ gda_mysql_connection_open_schema(Gda_MYSQL_Connection*      cnc,
     {
       g_warning("connection_open_schema: Unhandled SCHEMA_QTYPE %d\n", t);
     }
-  gda_mysql_recset_free(rc);
-  return 0;
+  return rc;
 }
 
 
 gint
 gda_mysql_connection_close(Gda_MYSQL_Connection* c)
 {
-  fprintf(stderr,"Calling mysql_close\n");
+  fprintf(stderr, "Calling mysql_close\n");
   mysql_close(c->mysql);
   return 0;
 }
-  
+
+gboolean
+gda_mysql_connection_supports (Gda_MYSQL_Connection *cnc, GDA_Connection_Feature feature)
+{
+  g_return_val_if_fail(cnc != 0, FALSE);
+
+  switch (feature)
+    {
+    case GDA_Connection_FEATURE_TRANSACTIONS : return FALSE;
+    case GDA_Connection_FEATURE_SEQUENCES : return TRUE; /* FIXME: ??? */
+    case GDA_Connection_FEATURE_INHERITANCE : return FALSE;
+    case GDA_Connection_FEATURE_PROCS : return FALSE;
+    default :
+      fprintf(stderr, "Invalid GDA_Connection_Feature %d", feature);
+    }
+  return FALSE;
+}
\ No newline at end of file
