@@ -23,6 +23,9 @@
 #include <glib.h>
 #include "gda-mysql-types.h"
 
+Gda_MYSQL_Error*          gda_mysql_error_new       (void);
+void                      gda_mysql_error_free      (Gda_MYSQL_Error* e);
+void                      gda_mysql_error_make      (Gda_MYSQL_Connection* cnc, gchar* where);
 
 
 #endif
