@@ -276,6 +276,11 @@ static CORBA_char *
 				 GDA_RowAttributes * columns,
 				 CORBA_Environment * ev);
 
+static CORBA_boolean
+ impl_GDA_Connection_supports(impl_POA_GDA_Connection *servant,
+                              GDA_Connection_Feature feature,
+                              CORBA_Environment * ev);
+
 static void impl_GDA_Recordset__destroy(impl_POA_GDA_Recordset * servant,
 					CORBA_Environment * ev);
 
@@ -542,6 +547,8 @@ static POA_GDA_Connection__epv impl_GDA_Connection_epv =
    (gpointer) & impl_GDA_Connection_stopLogging,
 
    (gpointer) & impl_GDA_Connection_createTable,
+   
+   (gpointer) & impl_GDA_Connection_supports
 
 };
 
@@ -707,7 +714,7 @@ make_errorbuffer(Gda_MYSQL_Connection* cnc)
   
   while(ptr)
     {
-      Gda_MYSQL_Error* e = ptr->data;
+      Gda_MYSQL_Error* e = (Gda_MYSQL_Error *) ptr->data;
       rc[idx].description = CORBA_string_dup(e->description);
       rc[idx].number = e->number;
       rc[idx].source = CORBA_string_dup(e->source);
@@ -716,7 +723,7 @@ make_errorbuffer(Gda_MYSQL_Connection* cnc)
       rc->_buffer[idx].helpctxt = CORBA_string_dup(e->helpctxt);
 #endif
       rc[idx].sqlstate = CORBA_string_dup(e->sqlstate);
-      rc[idx].nativeMsg   = CORBA_string_dup(e->native);
+      rc[idx].nativeMsg = CORBA_string_dup(e->native);
       gda_mysql_error_free(e);
       ptr = g_list_next(ptr);
       idx++;
@@ -1182,9 +1189,9 @@ CORBA_long
 impl_GDA_Connection_beginTransaction(impl_POA_GDA_Connection * servant,
 				     CORBA_Environment * ev)
 {
-   CORBA_long retval = 0;
+   CORBA_long retval = -1;
 
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
+   g_warning("%s not implemented", __PRETTY_FUNCTION__);
    return retval;
 }
 
@@ -1194,7 +1201,7 @@ impl_GDA_Connection_commitTransaction(impl_POA_GDA_Connection * servant,
 {
    CORBA_long retval = -1;
 
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
+   g_warning("%s not implemented", __PRETTY_FUNCTION__);
    return retval;
 }
 
@@ -1204,7 +1211,7 @@ impl_GDA_Connection_rollbackTransaction(impl_POA_GDA_Connection * servant,
 {
    CORBA_long retval = -1;
 
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
+   g_warning("%s not implemented", __PRETTY_FUNCTION__);
    return retval;
 }
 
@@ -1237,10 +1244,15 @@ impl_GDA_Connection_open(impl_POA_GDA_Connection * servant,
   fprintf(stderr, "gda_mysql_Connection_open: user   = '%s'\n", user);
   fprintf(stderr, "gda_mysql_Connection_open: passwd = '%s'\n", passwd);
   rc = gda_mysql_connection_open(servant->cnc, dsn, user, passwd);
-  fprintf(stderr,"gda_mysql_Connection_open: cnc = %p\n", servant->cnc);
-  fprintf(stderr,"                           cnc->mysql = %p\n", servant->cnc->mysql);
   if (rc < 0)
-    return -1;
+    {
+      GDA_DriverError* exception = GDA_DriverError__alloc();
+      exception->errors._length = g_list_length(servant->cnc->errors);
+      exception->errors._buffer = make_errorbuffer(servant->cnc);
+      exception->realcommand = CORBA_string_dup("Connection open");
+      CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_DriverError, exception);
+      return -1;
+    }
   else
     return 0;
 }
@@ -1260,7 +1272,14 @@ impl_GDA_Connection_openSchema(impl_POA_GDA_Connection * servant,
   memset(&e, '\0', sizeof(e));
 
   if ((recset = gda_mysql_connection_open_schema(servant->cnc, &e, t, constraints->_buffer, constraints->_length)) == 0)
-    return CORBA_OBJECT_NIL;
+    {
+      gchar bfr[128];
+      GDA_NotSupported* exception = GDA_NotSupported__alloc();
+      g_snprintf(bfr, sizeof(bfr), "SCHEMA type %d not supported\n", t);
+      exception->errormsg = CORBA_string_dup(bfr);
+      CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_NotSupported, exception);
+      return CORBA_OBJECT_NIL;
+    }
   new_recset = impl_GDA_Recordset__create(servant->poa, recset, ev);
   Exception(ev);
 
@@ -1301,7 +1320,7 @@ impl_GDA_Connection_listSources(impl_POA_GDA_Connection * servant,
 				CORBA_Environment * ev)
 {
   GDA_Connection_DSNlist *retval = 0;
-  g_error("%s is obsolete. Not Implemented", __PRETTY_FUNCTION__);
+  g_warning("%s is obsolete. Not Implemented", __PRETTY_FUNCTION__);
   return retval;
 }
 
@@ -1312,7 +1331,7 @@ impl_GDA_Connection_startLogging(impl_POA_GDA_Connection * servant,
 {
    CORBA_long retval = -1;
 
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
+   g_warning("%s not implemented", __PRETTY_FUNCTION__);
    return retval;
 }
 
@@ -1322,7 +1341,7 @@ impl_GDA_Connection_stopLogging(impl_POA_GDA_Connection * servant,
 {
    CORBA_long retval = -1;
 
-   g_error("%s not implemented", __PRETTY_FUNCTION__);
+   g_warning("%s not implemented", __PRETTY_FUNCTION__);
    return retval;
 }
 
@@ -1333,10 +1352,18 @@ impl_GDA_Connection_createTable(impl_POA_GDA_Connection * servant,
 				CORBA_Environment * ev)
 {
   CORBA_char *retval = 0;
-  g_error("%s not implemented", __PRETTY_FUNCTION__);
+  g_warning("%s not implemented", __PRETTY_FUNCTION__);
   return retval;
 }
 
+static CORBA_boolean
+impl_GDA_Connection_supports (impl_POA_GDA_Connection * servant,
+                              GDA_Connection_Feature feature,
+                              CORBA_Environment *ev)
+{
+  return gda_mysql_connection_supports(servant->cnc, feature);
+}
+
 static GDA_Recordset 
 impl_GDA_Recordset__create(PortableServer_POA poa, Gda_MYSQL_Recordset* rs, CORBA_Environment * ev)
 {
@@ -1711,7 +1738,7 @@ impl_GDA_Recordset_fetch(impl_POA_GDA_Recordset * servant,
   GList*                ptr;
   Gda_MYSQL_Field*      server_field;
 			 
-  fprintf(stderr,"gda_mysql_Recset_fetch called\n");
+  fprintf(stderr, "GDA_Recordset_fetch called\n");
   tmp_rows = 0;
   rowidx = 0;
   do
