@@ -27,14 +27,8 @@ void                   gda_odbc_cmd_free               (Gda_ODBC_Command* cmd);
      
 Gda_ODBC_Connection*   gda_odbc_cmd_set_connection     (Gda_ODBC_Command* cmd, Gda_ODBC_Connection* cnc);
 Gda_ODBC_Recordset*    gda_odbc_cmd_execute            (Gda_ODBC_Command* cmd,
-							Gda_ODBC_Error* error,
 							const GDA_CmdParameterSeq* params,
-							gulong
-							options);
-/*
-  Internal
-*/
-gint                  gda_odbc_gda2sqlctype            (GDA_ValueType tag);
-
+							gulong* affected_rows,
+							gulong options);
 #endif
 
