@@ -26,6 +26,22 @@ g_log_2_file(const gchar*       log_domain,
 
 
 
+static int
+Exception( CORBA_Environment* ev )
+{
+  switch( ev->_major )
+    {
+    case CORBA_SYSTEM_EXCEPTION:
+      g_warning("CORBA system exception %s.", CORBA_exception_id(ev));
+      return -1;
+    case CORBA_USER_EXCEPTION:
+      g_warning("CORBA user exception: %s.\n", CORBA_exception_id( ev ) );
+      return -1;
+    default:
+      break;
+    }
+  return 0;
+}
 
 int
 main(int argc, char* argv[])
