@@ -23,6 +23,9 @@
 Gda_ORACLE_Field* gda_oracle_field_new    (void);
 void              gda_oracle_field_free   (Gda_ORACLE_Field *);
 
+void              gda_oracle_field_sql2gdatype ( Gda_ORACLE_Field*,
+                                                 GDA_FieldAttributes*
+                                               );
 /*
  Debug
 */
