@@ -277,6 +277,11 @@ static CORBA_char *
 				 GDA_RowAttributes * columns,
 				 CORBA_Environment * ev);
 
+static CORBA_boolean 
+ impl_GDA_Connection_supports(impl_POA_GDA_Connection * servant,
+             GDA_Connection_Feature feature,
+				 CORBA_Environment * ev);
+
 static void impl_GDA_Recordset__destroy(impl_POA_GDA_Recordset * servant,
 					CORBA_Environment * ev);
 
@@ -544,6 +549,8 @@ static POA_GDA_Connection__epv impl_GDA_Connection_epv =
 
    (gpointer) & impl_GDA_Connection_createTable,
 
+   (gpointer) & impl_GDA_Connection_supports,
+
 };
 
 static PortableServer_ServantBase__epv impl_GDA_Recordset_base_epv =
@@ -1318,6 +1325,15 @@ impl_GDA_Connection_createTable(impl_POA_GDA_Connection * servant,
   return 0;
 }
 
+static CORBA_boolean 
+impl_GDA_Connection_supports(impl_POA_GDA_Connection * servant,
+            GDA_Connection_Feature feature,
+				CORBA_Environment * ev)
+{
+  Gda_ORACLE_Connection *cnc = servant->cnc;
+  return gda_oracle_connection_supports( cnc, feature );
+}
+
 static GDA_Recordset 
 impl_GDA_Recordset__create(PortableServer_POA poa, Gda_ORACLE_Recordset* recset, CORBA_Environment * ev)
 {
@@ -1796,7 +1812,7 @@ impl_GDA_Recordset_describe(impl_POA_GDA_Recordset * servant,
       field->definedSize = server_field->defined_length;
       
       field->scale       = server_field->num_scale;
-      field->gdaType     = GDA_TypeVarchar;
+      gda_oracle_field_sql2gdatype(server_field, field);
       field->nativeType  = server_field->sql_type;
       ptr = g_list_next(ptr);
       idx++;
