@@ -22,7 +22,7 @@
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkadjustment.h>
-#include <gda-recordset.h>
+#include <gda-connection.h>
 
 #define GDAFORM(obj)             GTK_CHECK_CAST(obj, gdaform_get_type(), Gdaform)
 #define GDAFORM_CLASS(klass)     GTK_CHECK_CLASS_CAST(klass, gdaform_get_type(), GdaformClass)
