@@ -90,11 +90,7 @@ main(int argc, char* argv[])
 				 GDA_Connection_GDCN_SCHEMA_TABLES,
 				 GDA_Connection_no_CONSTRAINT) == 0)
     fprintf(stderr,"gda_connection_open_schema failed\n");
-  if (gda_connection_close(cnc) < 0)
-    {
-      fprintf(stderr,"gda_connection_close failed\n");
-      return -1;
-    }
+  gda_connection_close(cnc);
   return 0;
 }
 
